(function() {
	let ExtensionOptionsMenu = {
		startup: function() {
			ExtensionOptionsMenu.register();
			ExtensionOptionsMenu.init();
			window.addEventListener('load', ExtensionOptionsMenu.setup, false);
			window.addEventListener('customizationchange', ExtensionOptionsMenu.setup, false);
		},

		register: function() {
			let EOMPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			this._branch = EOMPref.getBranch("extensions.ExtensionOptionsMenu.");
			this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
			this._branch.addObserver("", this, false);
		},

		unregister: function() {
			if (!this._branch)
				return;
			this._branch.removeObserver("", this);
		},

		observe: function(aSubject, aTopic, aData) {
			if(aTopic = "nsPref:changed")
				ExtensionOptionsMenu.setup();
		},

		init: function() {
			Components.utils.import("resource://gre/modules/AddonManager.jsm");
			AddonManager.addAddonListener(ExtensionOptionsMenu);
		},

		onDisabled: function() {
			ExtensionOptionsMenu.setup();
		},

		onEnabled: function() {
			ExtensionOptionsMenu.setup();
		},

		onUninstalled: function() {
			ExtensionOptionsMenu.setup();
		},

		onInstalled: function() {
			ExtensionOptionsMenu.setup();
		},

		setup: function() {
			let EOMPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);

			let EOToolsmenu = document.getElementById("extensionsoptionsmenu");
			EOToolsmenu.hidden = !EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.InToolsMenu");
			if (EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowIcons")) {
				EOToolsmenu.setAttribute("class", "menu-iconic");
				EOToolsmenu.setAttribute("style", "list-style-image: url(chrome://optionsmenu/skin/icon.png)");
			}
			else
				EOToolsmenu.setAttribute("class", "menu");

			ExtensionOptionsMenu.populate(EOToolsmenu.firstChild);

			let EOAppmenu = document.getElementById("extensionsoptionsAppmenu");
			if (EOAppmenu) {
				EOAppmenu.hidden = !EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.InAppMenu");
				document.getElementById("appmenu_addons").hidden = !EOAppmenu.hidden;

				ExtensionOptionsMenu.populate(EOAppmenu.firstChild);
			}

			let EOMToolbarButton = document.getElementById("extensionsoptionsmenutoolbar");
			if (EOMToolbarButton)
				ExtensionOptionsMenu.populate(EOMToolbarButton.firstChild);
		},

		populate: function(aParent) {
			let item, MenuIconURL, EOMShowIt = true, EOMBlackListArray = [];
			let EOMPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			let EOshowVersion = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowVersion");
			let EOshowIcon = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowIcons");
			let EOMshowAll = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowAll");
			let EOMshowDisabled = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowDisabled");
			let EOMshowIncompatible = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowIncompatible");
			let EOMquickActions = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.QuickActions");
			let EOMShowRestart = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowRestart"); 
			let EOMopenApp = window['BrowserOpenAddonsMgr'] || window['openAddonsMgr'] || window['toEM'];
			let EOMBlackList = EOMPref.getCharPref("extensions.ExtensionOptionsMenu.BlackList");
			if (EOMBlackList)
				EOMBlackListArray = EOMBlackList.split(",");

			AddonManager.getAllAddons(function(addon) {
				while (aParent.childNodes.length)
					aParent.removeChild(aParent.firstChild);

				addon.forEach(function(EOMaddon) {
					if (EOMaddon.type == "extension" && (!EOMaddon.appDisabled || (EOMaddon.appDisabled && EOMshowIncompatible)) && ((EOMaddon.isActive && EOMaddon.optionsURL) || ((EOMaddon.userDisabled && EOMshowDisabled) || (!EOMaddon.userDisabled && EOMshowAll) || (EOMaddon.appDisabled && EOMshowIncompatible)))) {
						item = document.createElement("menuitem");
						item.setAttribute("label", EOshowVersion ? EOMaddon.name + "  " + EOMaddon.version : EOMaddon.name);
						item.setAttribute("tooltiptext", EOMaddon.description);
						item.addEventListener("click", function(event) {
							event.preventDefault(); event.stopPropagation();
							if (event.button == 0 && EOMaddon.optionsURL)
								EOMaddon.optionsType == 2 ? EOMopenApp('addons://detail/' + encodeURIComponent(EOMaddon.id)) : openDialog(EOMaddon.optionsURL, EOMaddon.name, 'chrome,titlebar,toolbar,resizable,scrollbars,centerscreen,dialog=no,modal=no');
							if (EOMquickActions) {
								if (event.button == 1 && EOMaddon.homepageURL)
									try {
										gBrowser.addTab(EOMaddon.homepageURL);
									} catch(e) {
										Components.classes["@mozilla.org/messenger;1"].createInstance(Components.interfaces.nsIMessenger).launchExternalURL(EOMaddon.homepageURL);
									}
								if (event.button == 2)
									AddonManager.getAddonByID(EOMaddon.id, function(addon) {
										addon.userDisabled = !addon.userDisabled;
										if (addon.operationsRequiringRestart && !EOMShowRestart)
											Application.restart();
										ExtensionOptionsMenu.setup();
									});
							}
						}, false);
						if (!EOMaddon.optionsURL)
							item.setAttribute("disabled", true);
						if (!EOMaddon.isActive)
							item.setAttribute("EOMisDisabled", true);
						if (EOMaddon.pendingOperations == 1)
							item.setAttribute("EOMtoBeToggled", true);
						if (EOMaddon.pendingOperations == 2)
							item.setAttribute("EOMtoBeToggled", false);

						if (EOshowIcon) {
			 				item.setAttribute("class", "menuitem-iconic");
							MenuIconURL = EOMaddon.iconURL || 'chrome://optionsmenu/skin/icon.png';
							item.setAttribute("style", "list-style-image: url(" + MenuIconURL + ")");
						}
						else
							item.setAttribute("class", "menuitem");

						if (EOMBlackListArray)
							for (let i = 0; i < EOMBlackListArray.length; i++ ) {
								if (EOMBlackListArray[i].toLowerCase().replace(/^ | $/g,"") == EOMaddon.name.toLowerCase()) {
									EOMShowIt = false;
									break;
								}
								else
									EOMShowIt = true;
							}

						if (EOMShowIt)
							aParent.appendChild(item);
					}
				});

				Components.classes["@mozilla.org/xul/xul-sort-service;1"].getService(Components.interfaces.nsIXULSortService).sort(aParent, "label", "ascending");

				if (EOMShowRestart) {
					aParent.appendChild(document.createElement("menuseparator"));

					item = document.createElement("menuitem");
					item.setAttribute("label", document.getElementById("EOMstring-bundle").getString("EOMRestart"));
					item.addEventListener("command", function() { Application.restart() }, false);

					if (EOshowIcon) {
		 				item.setAttribute("class", "menuitem-iconic");
						item.setAttribute("style", "list-style-image: url(chrome://optionsmenu/skin/restart.png)");
					}
					else
						item.setAttribute("class", "menuitem");

					aParent.appendChild(item);
				}
			});
		}
	};
	ExtensionOptionsMenu.startup();
})()