	const {classes: Cc, interfaces: Ci, utils: Cu, results: Cr} = Components;
	
	const ADDON_ID = '{9aad3da6-6c46-4ef0-9109-6df5eaaf597c}';
	
	const Application       = Cc['@mozilla.org/fuel/application;1'].getService(Ci.fuelIApplication);
	const ObserverService   = Cc['@mozilla.org/observer-service;1'].getService(Ci.nsIObserverService);
	const PrefBranchService = Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefBranch);
	
	Cu.import('resource://gre/modules/XPCOMUtils.jsm');

	try{
		Cu.import('resource://gre/modules/AddonManager.jsm');
	}
	catch(ex){}
	
	
	var Uninstall = function(){};
	
	Uninstall.prototype = {
		classDescription : 'FirebugAutocompleter uninstaller',  
		contractID		 : '@firebugautocompleter.com/uninstall;1',  
		classID			 : Components.ID('{1012d640-2126-47aa-8c17-c08eb801925b}'),  
		QueryInterface	 : XPCOMUtils.generateQI([
			Ci.nsIObserver,
			Ci.nsISupports
		]),
		_xpcom_categories: [
		{ 
			category: 'app-startup',
			service : true 
		},
		{  
			category: 'profile-after-change',  
			service : true  
		}],  
		beingUninstalled: false,
		observe: function(aSubject,aTopic,aData){
			switch(aTopic){
				case 'app-startup':
				case 'profile-after-change':
					ObserverService.addObserver(this,'em-action-requested',false);
					ObserverService.addObserver(this,'quit-application-granted',false);
					try{
						AddonManager.addAddonListener(this);
					}
					catch(ex){}
				break;
				
				case 'em-action-requested':
					if(aSubject instanceof Ci.nsIUpdateItem && aSubject.id === ADDON_ID){
						switch(aData){
							case 'item-uninstalled':
								this.beingUninstalled = true;
							break;
							
							case 'item-cancel-action':
								this.beingUninstalled = false;
							break;
						}
					}
				break;
				
				case 'quit-application-granted':
					ObserverService.removeObserver(this,'em-action-requested');
					ObserverService.removeObserver(this,'quit-application-granted');
					try{
						AddonManager.removeAddonListener(this);
					}
					catch(ex){
					}
					
					if(this.beingUninstalled){
						this.deletePrefs();
					}
				break;
			}
		},
		onUninstalling: function(addon){
			if(addon.id === ADDON_ID){
				this.beingUninstalled = true;
			}
		},
		onOperationCancelled: function(addon){
			if(addon.id === ADDON_ID){
				this.beingUninstalled = false;
			}
		},
		deletePrefs: function(){
			try{
				PrefBranchService.deleteBranch('extensions.firebugautocompleter.');
			}
			catch(ex){}
		},
		consoleLog: function(str){
			var console = Application.console;
			console.open();
			if(typeof str === 'object'){
				for(var k in str){
					console.log(k + ' : ' + str[k]);
				};
			}
			console.log(str);
		}		
	};
	/**
	 * XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
	 * XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
	**/
	if(XPCOMUtils.generateNSGetFactory)
		var NSGetFactory = XPCOMUtils.generateNSGetFactory([Uninstall]);
	else
		var NSGetModule  = XPCOMUtils.generateNSGetModule([Uninstall]);