	const {classes: Cc, interfaces: Ci, utils: Cu, results: Cr} = Components;

	const XUL   = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
	const XHTML = 'http://www.w3.org/1999/xhtml';
	const XLINK = 'http://www.w3.org/1999/xlink';	
	
	const Application        = Cc['@mozilla.org/fuel/application;1'].getService(Ci.fuelIApplication);
	const AppShellService    = Cc['@mozilla.org/appshell/appShellService;1'].getService(Ci.nsIAppShellService);
	const ChromeRegistry     = Cc['@mozilla.org/chrome/chrome-registry;1'].getService(Ci.nsIChromeRegistry);
	const DebuggerService    = Cc['@mozilla.org/js/jsd/debugger-service;1'].getService(Ci.jsdIDebuggerService);
	const DirectoryService   = Cc['@mozilla.org/file/directory_service;1'].getService(Ci.nsIProperties);
	const IOService          = Cc['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);
	const LoaderService      = Cc['@mozilla.org/moz/jssubscript-loader;1'].getService(Ci.mozIJSSubScriptLoader);	
	const ObserverService    = Cc['@mozilla.org/observer-service;1'].getService(Ci.nsIObserverService);
	const PrefService 		 = Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefService);
	const PrefBranch         = Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefBranch);
	const PrefBranchInternal = Cc['@mozilla.org/preferences;1'].getService(Ci.nsIPrefBranchInternal);
	const StyleSheetService  = Cc['@mozilla.org/content/style-sheet-service;1'].getService(Ci.nsIStyleSheetService);
	const WindowMediator     = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
	
	var FileIOUtils = {
		read: function(file,charset){
			charset = (typeof charset === 'string') && charset || 'UTF-8';
			if(file instanceof Ci.nsIFile){
				var fileStream = Cc['@mozilla.org/network/file-input-stream;1'].createInstance(Ci.nsIFileInputStream);
				fileStream.init(file,1,0,false);
			}
			else
			if(file instanceof Ci.nsIInputStream){
				var fileStream = file;
			}
			else{
				return '';
			}
			var out = {};
			var converterStream = Cc['@mozilla.org/intl/converter-input-stream;1'].createInstance(Ci.nsIConverterInputStream);
			converterStream.init(fileStream,charset,fileStream.available(),converterStream.DEFAULT_REPLACEMENT_CHARACTER);
			converterStream.readString(fileStream.available(),out);
			var data = out.value;
			converterStream.close();
			fileStream.close();
			return data;
		},
		//https://gist.github.com/774684
		getChromeUrls: function(path){
			var leafNames = [];
			var uri = IOService.newURI(path, null, null);
			var baseURI = uri.clone().QueryInterface(Ci.nsIURL);
			baseURI.path = baseURI.directory;
			var localURI = ChromeRegistry.convertChromeURL(baseURI);
			if(localURI instanceof Ci.nsIFileURL){
				var dir = localURI.file;
				var files = dir.directoryEntries;
				while(files.hasMoreElements()){
					var file = files.getNext().QueryInterface(Ci.nsIFile);
					if(file.isFile())
						leafNames.push(file.leafName);
				};
			} 
			else 
			if(localURI instanceof Ci.nsIJARURI){
				var zip = this.openZipReader(localURI.JARFile);
			    try{
					var baseEntry = localURI.JAREntry;
					var pattern = baseEntry + '?*~' + baseEntry + '?*/*';
					var entries = zip.findEntries(pattern);
					while(entries.hasMore())
						leafNames.push(entries.getNext().substring(baseEntry.length));
				} 
				finally{
					zip.close();
				}
			}
			return leafNames;
		},
		openZipReader: function(uri){
			var zip = Cc['@mozilla.org/libjar/zip-reader;1'].createInstance(Ci.nsIZipReader);
			if(uri instanceof Ci.nsIFileURL){
				zip.open(uri.file);
			} 
			else 
			if(uri instanceof Ci.nsIJARURI){
				var innerZip = this.openZipReader(uri.JARFile);
				zip.openInner(innerZip,uri.JAREntry);
			}
			return zip;
		}
	};
	
	var ExtensionManagerUtils = {
		getInstallLocation: function(id){
			var file = DirectoryService.get('ProfD',Ci.nsIFile);
			file.append('extensions');
			file.append(id);
			if(file.exists() && !file.isDirectory() && !/\.xpi$/.test(file.leafName)){
				var data = FileIOUtils.read(file);
				file = Cc['@mozilla.org/file/local;1'].createInstance(Ci.nsILocalFile);  
				file.initWithPath(data);
			}
			else
			if(!file.exists()){
				file = null;
			}
			return file;
		},
		getAddon: function(id){
			var addon;
			if('extensions' in Application){
				addon = Application.extensions.get(id);
			}
			else{
				this.isFirefox4OrLater = true;
				Application.getExtensions(function(extensions){
					addon = extensions.get(id);
				});

				var thread = Cc['@mozilla.org/thread-manager;1'].getService().mainThread;
				while(addon === void(0)){
					thread.processNextEvent(true);
				};
			}
			return addon;
		},
		isFirefox4OrLater: false
	};
	
	const ADDON_ID 		= '{9aad3da6-6c46-4ef0-9109-6df5eaaf597c}';
	const ADDON 	    = ExtensionManagerUtils.getAddon(ADDON_ID);
	const ADDON_NAME    = ADDON.name;
	const ADDON_SLUG    = ADDON_NAME.replace(/\s+/g,'').toLowerCase();
	const ADDON_VERSION = ADDON.version;
	
	var PrefUtils = {
		getPref: function(name,d){
			var pref = PrefService.getBranch(null);
			if(d)
				pref = PrefService.getDefaultBranch(null);
			switch(pref.getPrefType(name)){
				case pref.PREF_STRING:
					var nsISupportsString = Ci.nsISupportsWString || Ci.nsISupportsString;
					return pref.getComplexValue(name, nsISupportsString).data;
					
				case pref.PREF_INT:
					return pref.getIntPref(name);
					
				case pref.PREF_BOOL:
					return pref.getBoolPref(name);
			}					
		},
		setPref: function(name, value){
			var pref = PrefService.getBranch(null);			
			switch(typeof value){
				case 'string':
					var nsISupportsString = Ci.nsISupportsWString || Ci.nsISupportsString;
					var string = ('@mozilla.org/supports-wstring;1' in Ci) ?
									Cc['@mozilla.org/supports-wstring;1'].createInstance(nsISupportsString):
									Cc['@mozilla.org/supports-string;1'].createInstance(nsISupportsString);
					string.data = value;
					pref.setComplexValue(name, nsISupportsString, string);
				break;
					
				case 'number':
					pref.setIntPref(name, parseInt(value));
				break;
					
				default:
					pref.setBoolPref(name, value);
				break;	
			}			
		},
		toggle: function(name){
			this.setPref(name,!this.getPref(name));
		},
		// http://piro.sakura.ne.jp/xul/tips/x0018.html
		addPrefListener: function(aObserver){
			try{
				aObserver.domains.forEach(function(d){
					PrefBranchInternal.addObserver(d, aObserver, false);	
				});
			}catch(e){}
		},
		removePrefListener: function(aObserver){
			try{
				aObserver.domains.forEach(function(d){
					PrefBranchInternal.removeObserver(d, aObserver);
				});
			}catch(e){}	
		}	
	}; 
	
	var TabUtils = {
		getTab: function(url){
			var t = null;
			var win = WindowMediator.getMostRecentWindow('navigator:browser');
			if(!win)
				return t;	
			var gb = win.gBrowser;	
			Array.forEach(gb.mTabs,function(tab){
				var b = tab.linkedBrowser;
				var spec = b.currentURI.spec.toLowerCase();
				if(typeof url === 'string'){
					url = url.toLowerCase();
					if(url === spec){
						t = tab;
					}
				}
				else{
					if(url.test(spec)){
						t = tab;
					}
				}
			});
			return t;
		},
		openInTab: function(url,mode,postData){
			if(typeof mode === 'undefined' || !/\d+/.test(mode)){
				mode = 1;
			}
			if(typeof postData === 'undefined')
				postData = null;
			var win = WindowMediator.getMostRecentWindow('navigator:browser');		
			var tab = this.getTab(url);
			if(tab){
				var win = WindowMediator.getMostRecentWindow('navigator:browser');
				win.gBrowser.selectedTab = tab;
				return;
			}
			switch(mode){
				case 0: // current
				default:
					win.loadURI(url,null,null);
				break;
			
				case 1: // foreground
					this.openInNewTab(url,false,postData);
				break;
			
				case 2: // background
					this.openInNewTab(url,true,postData);
				break;
			}
		},
		openInNewTab: function(url,background,postData){
			var win = WindowMediator.getMostRecentWindow('navigator:browser');
			if(win){		
				var tab = this.getTab(url);
				if(tab){
					win.gBrowser.selectedTab = tab;
					return;
				}
				if(typeof background === 'undefined')
					background = false;
				if(typeof postData === 'undefined')
					postData = null;
				win.gBrowser.loadOneTab(url,null,null,postData,background,false);
			}
			else{
				window.open(url);
			}
		},
		openHref: function(evt){
			var target = evt.target;
			if(!target.hasAttribute('href'))
				return;
			var url  = target.getAttribute('href');
			var mode = PrefUtils.getPref('extensions.firebugautocompleter.link.target');	
			this.openInTab(url,mode);
			evt.stopPropagation();
			evt.preventDefault();
		}
	};
	
	function OpenOptionWindow(){
		var theEM = WindowMediator.getMostRecentWindow('FirebugAutocompleter:Option');
		if(theEM){
			theEM.focus();
			return;
		}
		var win = WindowMediator.getMostRecentWindow('navigator:browser');
		if(win){
			var path = 'chrome://'+ADDON_SLUG+'/content/option/option.xul';
			win.openDialog(path,
								ADDON_SLUG,'chrome,centerscreen,toolbar,resizable=yes');		
		}
	};	
	
	function makeURI(url,charset,aUri){
		return IOService.newURI(url, charset, aUri);
	};
	
	function AutoSubScriptLoader(path){
		var self = this;
		var path = /^chrome:\/\//.test(path) && path || 
			path && 'chrome://'+ADDON_SLUG+'/content/' + path + '/' || 'chrome://'+ADDON_SLUG+'/content/';
		FileIOUtils.getChromeUrls(path).forEach(function(leafName){
			if(/\.js$/.test(leafName)){
				if(!/^overlay\./.test(leafName)){
					LoaderService.loadSubScript(path + leafName, self);
				}	
			}	
		});
	};
	function SubScriptLoader(js){
		var self = this;
		js.forEach(function(s){
			LoaderService.loadSubScript('chrome://'+ADDON_SLUG+'/content/' + s + '.js', self);
		});	
	};
	
	function Log(str){
		var console = Application.console;
		console.open();
		if(typeof str == 'object'){
			for(var k in str){
				console.log(k + ' : ' + str[k]);
			};
		}
		console.log(str);
	};

	 
	// Hatena Bookmark
	// https://addons.mozilla.org/ja/firefox/addon/11285/
	var EXPORTED_SYMBOLS = [
		m for(m in new Iterator(this, true))
			if(m !== 'EXPORTED_SYMBOLS')
	];	