/************************************************************************/
/*                                                                      */
/*      Tile Tabs  -  Firefox Extension  -  Main Window                 */
/*                                                                      */
/*      Javascript for Main Window overlay                              */
/*                                                                      */
/*      Copyright (C) 2010-2011  by  DW-dev                             */
/*                                                                      */
/*      Last Edit  -  16 Dec 2011                                       */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Notes on Tile Tabs Operation                                         */
/*                                                                      */
/* 1. The 'big idea' that makes this extension possible is applying     */
/*    'display:-moz-stack' to <tabpanels> within the <tabbrowser>       */
/*    and adjusting the size of the tab <notificationbox> elements      */
/*    by setting the four margin values in the style attribute.         */
/*                                                                      */
/* 2. The first tab in the first new or restored browser window does    */
/*    not fire a TabOpen event on start-up or restart. Any other tabs   */
/*    in the first new or restored browser window still fire TabOpen    */
/*    events on start-up or restart. None of the tabs in a second or    */
/*    subsequent window fire TabOpen events on start-up or restart.     */
/*                                                                      */
/* 3. The "tiletabs-assigned" attribute must be removed from the tab    */
/*    before calling unassignTab(), so that if removeTab() is called,   */
/*    then onTabClose() will do nothing. If the tab is the currently    */
/*    selected tab, then removeTab() is not called, which avoids the    */
/*    problem of tileTabs.prevTiledTab being invalid in onTabSelect().  */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Notes on Compatibility with Other Add-Ons                            */
/*                                                                      */
/* A. NoScript - Now compatible with NoScript.                          */
/*                                                                      */
/* B. Tab Mix Plus - The 'Mouse click (down and release) to select      */
/*    a tab' option must be set in Tab Mix Plus. When the 'Switch       */
/*    to last selected tab when clicking current one' option is also    */
/*    set, then clicking on the current tab does not work correctly.    */
/*    Workaround is to delay tab selection in onClickTab().             */
/*                                                                      */
/* C. FireGestures - Gestures operate on the currently selected tab.    */
/*    Workaround selects the target tab before operation starts.        */
/*    Only works properly with right-click gestures.                    */
/*                                                                      */
/* D. FireGestures - Gestures will often trigger a 'click' event.       */
/*    For a 'tab close' gesture, this results in onClickPanel()         */
/*    erroneously generating a 'TabSelect' event for the closing tab.   */
/*    Workaround stops onClickPanel() generating 'TabSelect' event.     */
/*    Only works properly with right-click gestures.                    */
/*                                                                      */
/************************************************************************/

var tileTabs =
{
    /********************************************************************/
    
    /* Shared variables */
    
    appInfo: Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo),
    versionComparator: Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator),
    ffVersion: "",
    
    runtime: Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime),
    osPlatform: "",
    
    prefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.tiletabs."),
    
    iosvc: Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService),
    secman: Components.classes["@mozilla.org/scriptsecuritymanager;1"].getService(Components.interfaces.nsIScriptSecurityManager),
    sstore: Components.classes["@mozilla.org/browser/sessionstore;1"].getService(Components.interfaces.nsISessionStore),
    
    old_BrowserToolboxCustomizeDone: null,
    
    tabPanels: null,
    extraBox: null,
    
    rootTile: null,
    panelCount: 0,
    splitterCount: 0,
    
    skinColor: "#5070C0",
    tileBorder: 2,
    splitterSize: 6,
    splitterSnap: 20,
    tabSingle: false,
    tabMove: false,
    mouseTile: 1,
    mouseAssign: 1,
    
    syncScroll: false,
    viewTiled: false,
    
    prevTiledTab: null,
    
    downTile: null,
    downSplitter: 0,
    lastScreenX: 0,
    lastScreenY: 0,
    moveTimeout: null,
    
    dragBrowser: null,
    dragTimeout: null,
    
    layoutType: 0,
    layoutString: "",
    
    debugTile: null,
    debugCount: 0,
    
    /********************************************************************/
    
    /* Tile object constructor */
    
    Tile: function()
    {
        this.parentTile = null;
        this.type = "panel";
        this.width = 0;
        this.height = 0;
        this.left = 0;
        this.top = 0;
        this.ratio = 1;
        /* Panel tile */
        this.panelID = "";
        this.scrollX = 0;
        this.scrollY = 0;
        /* Split tile */
        this.splitterID = new Array();
        this.childTile = new Array();
    },
    
    /********************************************************************/
    
    /* Listen for changes to settings */
    
    prefsObserver:
    {
        register: function()
        {
            /* Query nsIPrefBranch2 interface to observe changes to pref values */
            tileTabs.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
            
            /* Add the observer */
            tileTabs.prefs.addObserver("",this,false);
        },
        
        observe: function(subject,topic,data)
        {
            var i,j,key,keyset,menuitems,mainmenu,appmenu,appmenusep;
            var shortcuts = new Array("new","untile","expand","sync","view");
            var modifiers = new Array("","shift","control","control shift","alt","alt shift","alt control","alt control shift");
            
            if (topic != "nsPref:changed") return;
            
            tileTabs.skinColor = tileTabs.prefs.getCharPref("skin-color");
            tileTabs.tileBorder = tileTabs.prefs.getIntPref("tile-border");
            tileTabs.splitterSize = tileTabs.prefs.getIntPref("splitter-size");
            tileTabs.splitterSnap = tileTabs.prefs.getIntPref("splitter-snap");
            tileTabs.tabSingle = tileTabs.prefs.getBoolPref("tab-single");
            tileTabs.tabMove = tileTabs.prefs.getBoolPref("tab-move");
            tileTabs.mouseTile = tileTabs.prefs.getIntPref("mouse-tile");
            tileTabs.mouseAssign = tileTabs.prefs.getIntPref("mouse-assign");
            
            for (i = 0; i < 5; i++)
            {
                key = document.getElementById("tiletabs-" + shortcuts[i] + "-key");
                if ((tileTabs.prefs.getIntPref("fkey-" + shortcuts[i]) & 0x0F) != 0)
                {
                    key.setAttribute("disabled","false");
                    key.setAttribute("keycode","VK_F"+(tileTabs.prefs.getIntPref("fkey-" + shortcuts[i]) & 0x0F));
                    key.setAttribute("modifiers",modifiers[tileTabs.prefs.getIntPref("fkey-" + shortcuts[i]) >> 4]);
                }
                else
                {
                    key.setAttribute("disabled","true");
                    key.removeAttribute("keycode");
                    key.removeAttribute("modifiers");
                }
            }
            
            keyset = document.getElementById("tiletabs-keyset");
            keyset.parentNode.insertBefore(keyset,keyset.nextSibling);  
            for (i = 0; i < 5; i++)
            {
                menuitems = document.getElementsByAttribute("key","tiletabs-" + shortcuts[i] + "-key");
                for (j = 0; j < menuitems.length; j++)
                {
                    menuitems[j].setAttribute("acceltext","");
                    menuitems[j].removeAttribute("acceltext");
                }
            }
            
            mainmenu = document.getElementById("tiletabs-mainmenu");
            mainmenu.setAttribute("hidden",!tileTabs.prefs.getBoolPref("menu-tilemenu"));
            
            if (tileTabs.ffVersion >= "4.0")
            {
                if (document.getElementById("appmenu-button") != null)
                {
                    appmenu = document.getElementById("tiletabs-appmenu");
                    appmenu.setAttribute("hidden",!tileTabs.prefs.getBoolPref("menu-tilemenu"));
                    appmenusep = document.getElementById("tiletabs-appmenu-separator");
                    appmenusep.setAttribute("hidden",!tileTabs.prefs.getBoolPref("menu-tilemenu"));
                }
            }
            
            for (i = 0; i < document.styleSheets.length; i++)
            {
                if (document.styleSheets[i].href == "chrome://tiletabs/skin/tiletabs-buttonicon.css")
                    document.styleSheets[i].cssRules[0].style.setProperty("background-color",tileTabs.skinColor,"");
                if (document.styleSheets[i].href == "chrome://tiletabs/skin/tiletabs-menuicon.css")
                    document.styleSheets[i].cssRules[0].style.setProperty("background-color",tileTabs.skinColor,"");
            }
            
            if (tileTabs.panelCount > 0) tileTabs.resizeTile(tileTabs.rootTile,tileTabs.tabPanels.boxObject.width,tileTabs.tabPanels.boxObject.height,0,0);

            if (tileTabs.viewTiled && tileTabs.prefs.getBoolPref("tab-assignselect")) gBrowser.selectedTab = tileTabs.prevTiledTab;
            
            if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
            {
                tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                tileTabs.highlightPanel();
            }
            else
            {
                tileTabs.drawSelectedPanel();
            }
            tileTabs.styleTiledTabs();
            tileTabs.moveTiledTabs();
        }
    },
    
    /********************************************************************/
    
    /* Listen for changes to location on all tabs */
    
    progressListener:
    {
        QueryInterface: function(aIID)
        {
            if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
                aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
                aIID.equals(Components.interfaces.nsISupports))
              return this;
            throw Components.results.NS_NOINTERFACE;
        },
        
        onLocationChange: function(browser,webProgress,request,URI)
        {
            if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
            {
                if (gBrowser.selectedTab.linkedBrowser.currentURI.spec == "about:addons")
                    document.getElementById("main-window").removeAttribute("disablechrome");
            }
        },
        
        onProgressChange: function(browser,webProgress,request,curSelf,maxSelf,curTot,maxTot)
        {
        },
        
        onSecurityChange: function(browser,webProgress,request,state)
        {
        },
        
        onStateChange: function(browser,webProgress,request,flags,status)
        {
        },
        
        onStatusChange: function(browser,webProgress,request,status,message)
        {
        }        
    },
    
    /********************************************************************/
    
    /* On Load functions */
    
    onLoad: function()
    {
        /* Determine Firefox version */
        
        if (tileTabs.versionComparator.compare(tileTabs.appInfo.version,"8.0b1") >= 0) tileTabs.ffVersion = "8.0";
        else if (tileTabs.versionComparator.compare(tileTabs.appInfo.version,"4.0b1") >= 0) tileTabs.ffVersion = "4.0";
        else tileTabs.ffVersion = "3.6";
        
        document.getElementById("main-window").setAttribute("tiletabs-ffversion",tileTabs.ffVersion);
        
        /* Look for presence of Tile View */
        
        window.setTimeout(function ()
        {
            if (document.getElementById("main-window").hasAttribute("tileview-ffversion")) tileTabs.onLoadAbandon();
            else tileTabs.onLoadContinue();
        },100);
        
        /* Redefine BrowserToolboxCustomizeDone - must be done before break in execution */
        
        tileTabs.old_BrowserToolboxCustomizeDone = BrowserToolboxCustomizeDone;
        
        BrowserToolboxCustomizeDone = function new_BrowserToolboxCustomizeDone(aToolboxChanged)
        {
            var buttonmenu;
            
            /* Call original BrowserToolboxCustomizeDone */
            
            tileTabs.old_BrowserToolboxCustomizeDone(aToolboxChanged);
            
            /* Update button state and add listener for button click when Customize Toolbar window closed */
            
            if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
            {
                buttonmenu.setAttribute("layout",(tileTabs.panelCount > 0));
                document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").setAttribute("checked",tileTabs.viewTiled);
                document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").addEventListener("click",tileTabs.onClickButton,false);
            }
        }
    },
    
    onLoadAbandon: function()
    {
        var buttonmenu;
        
        /* Hide user interface */
        
        document.getElementById("tiletabs-keyset").setAttribute("disabled","true");
        
        document.getElementById("tiletabs-mainmenu").setAttribute("hidden","true");
        
        if (tileTabs.ffVersion >= "4.0")
        {
            if (document.getElementById("appmenu-button") != null)
            {
                document.getElementById("tiletabs-appmenu").setAttribute("hidden","true");
                document.getElementById("tiletabs-appmenu-separator").setAttribute("hidden","true");
            }
        }
        
        document.getElementById("tiletabs-tabmenu-template").setAttribute("hidden","true");
        
        document.getElementById("tiletabs-contentmenu-template").setAttribute("hidden","true");
        
        if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
            buttonmenu.setAttribute("hidden","true");
    },
    
    onLoadContinue: function()
    {
        var i,j,tab,notificationBox,key,keyset,menuitems,mainmenu,appmenu,appmenusep,navbar,urlbar,searchbar,menuTemplate,menuPopup,buttonmenu,detector,browser;
        var shortcuts = new Array("new","untile","expand","sync","view");
        var modifiers = new Array("","shift","control","control shift","alt","alt shift","alt control","alt control shift");
        
        /* Determine operating system platform */
        
        tileTabs.osPlatform = tileTabs.runtime.OS;
        
        /* Initialise global constants */
        
        tab = gBrowser.tabContainer.childNodes[0];
        notificationBox = document.getElementById(tab.linkedPanel);
        tileTabs.tabPanels = notificationBox.parentNode;
        tileTabs.extraBox = document.getElementById("tiletabs-extrabox");
        
        /* Register preferences observer */
        
        tileTabs.prefsObserver.register();
        
        /* Load option settings */
        
        tileTabs.skinColor = tileTabs.prefs.getCharPref("skin-color");
        tileTabs.tileBorder = tileTabs.prefs.getIntPref("tile-border");
        tileTabs.splitterSize = tileTabs.prefs.getIntPref("splitter-size");
        tileTabs.splitterSnap = tileTabs.prefs.getIntPref("splitter-snap");
        tileTabs.tabSingle = tileTabs.prefs.getBoolPref("tab-single");
        tileTabs.tabMove = tileTabs.prefs.getBoolPref("tab-move");
        tileTabs.mouseTile = tileTabs.prefs.getIntPref("mouse-tile");
        tileTabs.mouseAssign = tileTabs.prefs.getIntPref("mouse-assign");
        
        for (i = 0; i < 5; i++)
        {
            key = document.getElementById("tiletabs-" + shortcuts[i] + "-key");
            if ((tileTabs.prefs.getIntPref("fkey-" + shortcuts[i]) & 0x0F) != 0)
            {
                key.setAttribute("disabled","false");
                key.setAttribute("keycode","VK_F"+(tileTabs.prefs.getIntPref("fkey-" + shortcuts[i]) & 0x0F));
                key.setAttribute("modifiers",modifiers[tileTabs.prefs.getIntPref("fkey-" + shortcuts[i]) >> 4]);
            }
            else
            {
                key.setAttribute("disabled","true");
                key.removeAttribute("keycode");
                key.removeAttribute("modifiers");
            }
        }
        
        keyset = document.getElementById("tiletabs-keyset");
        keyset.parentNode.insertBefore(keyset,keyset.nextSibling);
        for (i = 0; i < 5; i++)
        {
            menuitems = document.getElementsByAttribute("key","tiletabs-" + shortcuts[i] + "-key");
            for (j = 0; j < menuitems.length; j++)
            {
                menuitems[j].setAttribute("acceltext","");
                menuitems[j].removeAttribute("acceltext");
            }
        }
        
        mainmenu = document.getElementById("tiletabs-mainmenu");
        mainmenu.setAttribute("hidden",!tileTabs.prefs.getBoolPref("menu-tilemenu"));
        
        if (tileTabs.ffVersion >= "4.0")
        {
            if (document.getElementById("appmenu-button") != null)
            {
                appmenu = document.getElementById("tiletabs-appmenu");
                appmenu.setAttribute("hidden",!tileTabs.prefs.getBoolPref("menu-tilemenu"));
                appmenusep = document.getElementById("tiletabs-appmenu-separator");
                appmenusep.setAttribute("hidden",!tileTabs.prefs.getBoolPref("menu-tilemenu"));
            }
        }
        
        /* Add button to navigation toolbar on first run after installation */
        
        if (tileTabs.prefs.getBoolPref("install-firstrun"))
        {
            if (document.getElementById("tiletabs-buttonmenu") == null) /* button not on toolbar */
            {
                navbar = document.getElementById("nav-bar");
                navbar.setAttribute("collapsed","false");
                urlbar = document.getElementById("urlbar-container");
                searchbar = document.getElementById("search-container");
                if (urlbar != null && urlbar.parentNode == navbar) navbar.insertItem("tiletabs-buttonmenu",urlbar,null,false);
                else if (searchbar != null && searchbar.parentNode == navbar) navbar.insertItem("tiletabs-buttonmenu",searchbar,null,false);
                else navbar.insertItem("tiletabs-buttonmenu",null,null,false);
                navbar.setAttribute("currentset",navbar.currentSet);
                document.persist("nav-bar","currentset");
            }
            tileTabs.prefs.setBoolPref("install-firstrun",false)
        }
        
        /* Modify style sheets for button and menu icon background colors */
        
        for (i = 0; i < document.styleSheets.length; i++)
        {
            if (document.styleSheets[i].href == "chrome://tiletabs/skin/tiletabs-buttonicon.css")
                document.styleSheets[i].cssRules[0].style.setProperty("background-color",tileTabs.skinColor,"");
            if (document.styleSheets[i].href == "chrome://tiletabs/skin/tiletabs-menuicon.css")
                document.styleSheets[i].cssRules[0].style.setProperty("background-color",tileTabs.skinColor,"");
        }
        
        /* Create tab context menu from template - has to be done like this for Firefox 3.6 */
        
        menuTemplate = document.getElementById("tiletabs-tabmenu-template");
        if (tileTabs.ffVersion == "3.6") menuPopup = document.getAnonymousElementByAttribute(gBrowser,"anonid","tabContextMenu");
        else if (tileTabs.ffVersion >= "4.0") menuPopup = document.getElementById("tabContextMenu");
        while (menuTemplate.childNodes.length > 0)
        {
            menuPopup.appendChild(menuTemplate.childNodes[0].cloneNode(true));
            menuTemplate.removeChild(menuTemplate.childNodes[0]);
        }
        
        /* Add listener for button click */
        
        if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
            document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").addEventListener("click",tileTabs.onClickButton,false);
        
        /* Add listeners for popup menus - except button menu */
        
        menuPopup = document.getElementById("tiletabs-mainmenu-popup");
        menuPopup.addEventListener("popupshowing",tileTabs.onPopupShowingMainMenu,false);
        
        if (tileTabs.ffVersion >= "4.0")
        {
            if (document.getElementById("appmenu-button") != null)
            {
                menuPopup = document.getElementById("tiletabs-appmenu-popup");
                menuPopup.addEventListener("popupshowing",tileTabs.onPopupShowingAppMenu,false);
            }
        }
        
        if (tileTabs.ffVersion == "3.6") menuPopup = document.getAnonymousElementByAttribute(gBrowser,"anonid","tabContextMenu");
        else if (tileTabs.ffVersion >= "4.0") menuPopup = document.getElementById("tabContextMenu");
        menuPopup.addEventListener("popupshowing",tileTabs.onPopupShowingTabMenu,false);
        
        menuPopup = document.getElementById("contentAreaContextMenu");
        menuPopup.addEventListener("popupshowing",tileTabs.onPopupShowingContentMenu,false);
        
        /* Add listeners for tab events */
        
        gBrowser.tabContainer.addEventListener("TabOpen",tileTabs.onTabOpen,false);
        gBrowser.tabContainer.addEventListener("TabSelect",tileTabs.onTabSelect,false);
        gBrowser.tabContainer.addEventListener("TabClose",tileTabs.onTabClose,false);
        gBrowser.tabContainer.addEventListener("TabMove",tileTabs.onTabMove,false);
        gBrowser.tabContainer.addEventListener("TabShow",tileTabs.onTabShowHide,false);
        gBrowser.tabContainer.addEventListener("TabHide",tileTabs.onTabShowHide,false);
        
        /* Add progress listener for all tabs */
        
        gBrowser.addTabsProgressListener(tileTabs.progressListener);
        
        /* Create iframe to detect changes in size of tabpanels */
        
        detector = document.createElement("iframe");
        detector.setAttribute("id","tiletabs-detector");
        detector.setAttribute("src","about:blank");
        detector.setAttribute("flex","1");
        tileTabs.extraBox.appendChild(detector);
        
        detector.addEventListener("MozScrolledAreaChanged",tileTabs.onDetectorAreaChanged,false);
        detector.addEventListener("load",tileTabs.onDetectorLoaded,true);
        
        /* Do same as onTabOpen() for tabs that do not fire TabOpen events */
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)  /* see Note 2 */
        {
            tab = gBrowser.tabContainer.childNodes[i];
            
            tab.addEventListener("mousedown",tileTabs.onMouseDownTab,true);
            tab.addEventListener("click",tileTabs.onClickTab,true);
            
            browser = tab.linkedBrowser;
            browser.addEventListener("dragstart",tileTabs.onDragStartBrowser,true);
            browser.addEventListener("dragover",tileTabs.onDragOverBrowser,true);
            browser.addEventListener("drop",tileTabs.onDropBrowser,true);
        }
        
        tileTabs.prevTiledTab = gBrowser.selectedTab;
        
        tileTabs.hideNonSelectedPanels();
        tileTabs.drawSelectedPanel();
    },
    
    onDetectorLoaded: function(event)
    {
        var detector,buttonmenu;
        
        detector = event.currentTarget;
        detector.removeEventListener("load",tileTabs.onDetectorLoaded,true);
        
        /* Open restore or default tile layout */
        
        if (tileTabs.prefs.getBoolPref("startup-restore") && tileTabs.openRestoreLayout())
        {
            tileTabs.prevTiledTab = gBrowser.selectedTab;
            
            tileTabs.viewTiled = true;
            
            if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
            {
                buttonmenu.setAttribute("layout","true");
                document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").setAttribute("checked","true");
            }
            
            if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
            {
                tileTabs.hideNonTiledPanels();
                tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                tileTabs.highlightPanel();
            }
            else
            {
                tileTabs.hideNonSelectedPanels();
                tileTabs.drawSelectedPanel();
            }
            tileTabs.styleTiledTabs();
            tileTabs.moveTiledTabs();
        }
        else if (tileTabs.prefs.getBoolPref("startup-default"))
        {
            tileTabs.menuActions('new-default',null);
        }
    },
    
    /********************************************************************/
    
    /* Add tile functions */
    
    addTileFirst: function(firstTab)
    {
        var notificationBox,browser;
        
        tileTabs.rootTile = new tileTabs.Tile;
        tileTabs.rootTile.panelID = firstTab.linkedPanel;
        tileTabs.rootTile.width = tileTabs.tabPanels.boxObject.width;
        tileTabs.rootTile.height = tileTabs.tabPanels.boxObject.height;
        
        /* Add listeners for firstTab.linkedPanel and associated browser */
        
        notificationBox = document.getElementById(firstTab.linkedPanel);
        notificationBox.addEventListener("click",tileTabs.onClickPanel,false);
        notificationBox.addEventListener("mousedown",tileTabs.onMouseDownPanel,false);
        
        browser = firstTab.linkedBrowser;
        browser.addEventListener("scroll",tileTabs.onScrollBrowser,false);
        
        /* Set tile attribute for firstTab */
        
        firstTab.setAttribute("tiletabs-assigned","true");
        
        tileTabs.panelCount++;
    },
    
    addTile: function(tile,newTab,direction)
    {
        var i,pc,splitterBox,splitterName,menuPopup,notificationBox,browser;
        
        /* If 'tile.type' is 'panel', then 'direction' can be any direction */
        /* If 'tile.type' is 'vsplit', then 'direction' must be 'left' or 'right' */
        /* If 'tile.type' is 'hsplit', then 'direction' must be 'above' or 'below' */
        
        if (tile.parentTile == null && tile.type != "panel")
        {
            tileTabs.rootTile = new tileTabs.Tile;
            tileTabs.rootTile.type = (direction == "left" || direction == "right") ? "vsplit" : "hsplit";
            tileTabs.rootTile.width = tileTabs.tabPanels.boxObject.width;
            tileTabs.rootTile.height = tileTabs.tabPanels.boxObject.height;
            tileTabs.rootTile.childTile[0] = tile;
            tile.parentTile = tileTabs.rootTile;
        }
        
        if (tile.parentTile != null &&
             ( (tile.parentTile.type == "vsplit" && (direction == "left" || direction == "right")) ||
               (tile.parentTile.type == "hsplit" && (direction == "above" || direction == "below")) ) )   /* add sibling panel tile to parent tile */
        {
            for (pc = 0; pc < tile.parentTile.childTile.length; pc++) if (tile.parentTile.childTile[pc] == tile) break;
            tile = tile.parentTile;
            
            switch (direction)
            {
                case "left":
                    for (i = tile.childTile.length-1; i >= pc; i--)
                    {
                        tile.childTile[i+1] = tile.childTile[i];
                        if (i > pc) tile.splitterID[i] = tile.splitterID[i-1];
                    }
                    tile.childTile[pc] = new tileTabs.Tile;
                    tile.childTile[pc].parentTile = tile;
                    tile.childTile[pc].width = Math.floor((tile.childTile[pc+1].width-tileTabs.splitterSize)/2);
                    tile.childTile[pc].height = tile.height;
                    tile.childTile[pc].left = tile.childTile[pc+1].left;
                    tile.childTile[pc].top = tile.top;
                    tile.childTile[pc].panelID = newTab.linkedPanel;
                    tile.childTile[pc+1].left = tile.childTile[pc+1].left+tileTabs.splitterSize+Math.floor((tile.childTile[pc+1].width-tileTabs.splitterSize)/2);
                    tile.childTile[pc+1].width = tile.childTile[pc+1].width-tileTabs.splitterSize-Math.floor((tile.childTile[pc+1].width-tileTabs.splitterSize)/2);
                    tile.splitterID[pc] = splitterName = "tiletabs-splitter" + tileTabs.splitterCount++;
                    for (i = 0; i < tile.childTile.length; i++)
                        tile.childTile[i].ratio = tile.childTile[i].width/(tile.width-tile.splitterID.length*tileTabs.splitterSize);
                    if (tile.childTile[pc+1].type != "panel")
                        tileTabs.resizeTile(tile.childTile[pc+1],tile.childTile[pc+1].width,tile.height,tile.childTile[pc+1].left,tile.top);
                    break;
                
                case "right":
                    for (i = tile.childTile.length-1; i > pc; i--)
                    {
                        tile.childTile[i+1] = tile.childTile[i];
                        tile.splitterID[i] = tile.splitterID[i-1];
                    }
                    tile.childTile[pc+1] = new tileTabs.Tile;
                    tile.childTile[pc+1].parentTile = tile;
                    tile.childTile[pc+1].width = tile.childTile[pc].width-tileTabs.splitterSize-Math.floor((tile.childTile[pc].width-tileTabs.splitterSize)/2);
                    tile.childTile[pc+1].height = tile.height;
                    tile.childTile[pc+1].left = tile.childTile[pc].left+tileTabs.splitterSize+Math.floor((tile.childTile[pc].width-tileTabs.splitterSize)/2);
                    tile.childTile[pc+1].top = tile.top;
                    tile.childTile[pc+1].panelID = newTab.linkedPanel;
                    tile.childTile[pc].width = Math.floor((tile.childTile[pc].width-tileTabs.splitterSize)/2);
                    tile.splitterID[pc] = splitterName = "tiletabs-splitter" + tileTabs.splitterCount++;
                    for (i = 0; i < tile.childTile.length; i++)
                        tile.childTile[i].ratio = tile.childTile[i].width/(tile.width-tile.splitterID.length*tileTabs.splitterSize);
                    if (tile.childTile[pc].type != "panel")
                        tileTabs.resizeTile(tile.childTile[pc],tile.childTile[pc].width,tile.height,tile.childTile[pc].left,tile.top);
                    break;
                
                case "above":
                    for (i = tile.childTile.length-1; i >= pc; i--)
                    {
                        tile.childTile[i+1] = tile.childTile[i];
                        if (i > pc) tile.splitterID[i] = tile.splitterID[i-1];
                    }
                    tile.childTile[pc] = new tileTabs.Tile;
                    tile.childTile[pc].parentTile = tile;
                    tile.childTile[pc].width = tile.width;
                    tile.childTile[pc].height = Math.floor((tile.childTile[pc+1].height-tileTabs.splitterSize)/2);
                    tile.childTile[pc].left = tile.left;
                    tile.childTile[pc].top = tile.childTile[pc+1].top;
                    tile.childTile[pc].panelID = newTab.linkedPanel;
                    tile.childTile[pc+1].top = tile.childTile[pc+1].top+tileTabs.splitterSize+Math.floor((tile.childTile[pc+1].height-tileTabs.splitterSize)/2);
                    tile.childTile[pc+1].height = tile.childTile[pc+1].height-tileTabs.splitterSize-Math.floor((tile.childTile[pc+1].height-tileTabs.splitterSize)/2);
                    tile.splitterID[pc] = splitterName = "tiletabs-splitter" + tileTabs.splitterCount++;
                    for (i = 0; i < tile.childTile.length; i++)
                        tile.childTile[i].ratio = tile.childTile[i].height/(tile.height-tile.splitterID.length*tileTabs.splitterSize);
                    if (tile.childTile[pc+1].type != "panel")
                        tileTabs.resizeTile(tile.childTile[pc+1],tile.width,tile.childTile[pc+1].height,tile.left,tile.childTile[pc+1].top);
                    break;
                
                case "below":
                    for (i = tile.childTile.length-1; i > pc; i--)
                    {
                        tile.childTile[i+1] = tile.childTile[i];
                        tile.splitterID[i] = tile.splitterID[i-1];
                    }
                    tile.childTile[pc+1] = new tileTabs.Tile;
                    tile.childTile[pc+1].parentTile = tile;
                    tile.childTile[pc+1].width = tile.width;
                    tile.childTile[pc+1].height = tile.childTile[pc].height-tileTabs.splitterSize-Math.floor((tile.childTile[pc].height-tileTabs.splitterSize)/2);
                    tile.childTile[pc+1].left = tile.left;
                    tile.childTile[pc+1].top = tile.childTile[pc].top+tileTabs.splitterSize+Math.floor((tile.childTile[pc].height-tileTabs.splitterSize)/2);
                    tile.childTile[pc+1].panelID = newTab.linkedPanel;
                    tile.childTile[pc].height = Math.floor((tile.childTile[pc].height-tileTabs.splitterSize)/2);
                    tile.splitterID[pc] = splitterName = "tiletabs-splitter" + tileTabs.splitterCount++;
                    for (i = 0; i < tile.childTile.length; i++)
                        tile.childTile[i].ratio = tile.childTile[i].height/(tile.height-tile.splitterID.length*tileTabs.splitterSize);
                    if (tile.childTile[pc].type != "panel")
                        tileTabs.resizeTile(tile.childTile[pc],tile.width,tile.childTile[pc].height,tile.left,tile.childTile[pc].top);
                    break;
            }
        }
        else /* split this panel tile into two child panel tiles */
        {
            tile.childTile[0] = new tileTabs.Tile;
            tile.childTile[0].parentTile = tile;
            tile.childTile[0].width = (direction == "left" || direction == "right") ? Math.floor((tile.width-tileTabs.splitterSize)/2) : tile.width;
            tile.childTile[0].height = (direction == "left" || direction == "right") ? tile.height : Math.floor((tile.height-tileTabs.splitterSize)/2);
            tile.childTile[0].left = tile.left;
            tile.childTile[0].top = tile.top;
            tile.childTile[0].ratio = 1/2;
            tile.childTile[0].panelID = (direction == "left" || direction == "above") ? newTab.linkedPanel : tile.panelID;
            tile.childTile[1] = new tileTabs.Tile;
            tile.childTile[1].parentTile = tile;
            tile.childTile[1].width = (direction == "left" || direction == "right") ? tile.width-tileTabs.splitterSize-tile.childTile[0].width : tile.width;
            tile.childTile[1].height = (direction == "left" || direction == "right") ? tile.height : tile.height-tileTabs.splitterSize-tile.childTile[0].height;
            tile.childTile[1].left = (direction == "left" || direction == "right") ? tile.left+tileTabs.splitterSize+tile.childTile[0].width : tile.left;
            tile.childTile[1].top = (direction == "left" || direction == "right") ? tile.top : tile.top+tileTabs.splitterSize+tile.childTile[0].height;
            tile.childTile[1].ratio = 1/2;
            tile.childTile[1].panelID = (direction == "left" || direction == "above") ? tile.panelID : newTab.linkedPanel;
            tile.type = (direction == "left" || direction == "right") ? "vsplit" : "hsplit";
            tile.panelID = "";
            tile.splitterID[0] = splitterName = "tiletabs-splitter" + tileTabs.splitterCount++;
        }
        
        /* Create splitter */
        
        splitterBox = (direction == "left" || direction == "right") ? document.createElement("vbox") : document.createElement("hbox");
        splitterBox.setAttribute("id",splitterName);
        splitterBox.setAttribute("context","tiletabs-splittermenu-popup");
        tileTabs.extraBox.appendChild(splitterBox);
        
        /* Add listeners for splitter */
        
        splitterBox.addEventListener("dblclick",tileTabs.onDblClickSplitter,false);
        splitterBox.addEventListener("mousedown",tileTabs.onMouseDownSplitter,false);
        
        /* Add listeners for popup menu */
        
        menuPopup = document.getElementById("tiletabs-splittermenu-popup");
        menuPopup.addEventListener("popupshowing",tileTabs.onPopupShowingSplitterMenu,false);
        menuPopup.addEventListener("popuphiding",tileTabs.onPopupHidingSplitterMenu,false);
        
        /* Add listeners for newTab.linkedPanel and associated browser */
        
        notificationBox = document.getElementById(newTab.linkedPanel);
        notificationBox.addEventListener("click",tileTabs.onClickPanel,false);
        notificationBox.addEventListener("mousedown",tileTabs.onMouseDownPanel,false);
        
        browser = newTab.linkedBrowser;
        browser.addEventListener("scroll",tileTabs.onScrollBrowser,false);
        
        /* Set tile attribute for newTab */
        
        newTab.setAttribute("tiletabs-assigned","true");
        
        tileTabs.panelCount++;
    },
    
    /********************************************************************/
    
    /* Remove tile function */
    
    removeTile: function(tile,select)
    {
        var i,num,pc,gpc,gptile,ptile,stile,tab,splitterBox,notificationBox,browser;
        
        ptile = tile.parentTile;
        for (pc = 0; pc < ptile.childTile.length; pc++) if (ptile.childTile[pc] == tile) break;
        if (pc == 0) stile = ptile.childTile[1]; else stile = ptile.childTile[pc-1];  /* sibling tile */
        
        /* Remove tile, splitter and listeners */
        if (pc == 0) splitterBox = document.getElementById(ptile.splitterID[0]);
        else splitterBox = document.getElementById(ptile.splitterID[pc-1]);
        splitterBox.removeEventListener("dblclick",tileTabs.onDblClickSplitter,false);
        splitterBox.removeEventListener("mousedown",tileTabs.onMouseDownSplitter,false);
        splitterBox.removeEventListener("popupshowing",tileTabs.onPopupShowingSplitterMenu,false);
        splitterBox.removeEventListener("popuphiding",tileTabs.onPopupHidingSplitterMenu,false);
        tileTabs.extraBox.removeChild(splitterBox);
        notificationBox = document.getElementById(tile.panelID);
        notificationBox.removeEventListener("click",tileTabs.onClickPanel,false);
        notificationBox.removeEventListener("mousedown",tileTabs.onMouseDownPanel,false);
        tab = tileTabs.findTabByPanel(tile.panelID)    
        browser = tab.linkedBrowser;
        browser.removeEventListener("scroll",tileTabs.onScrollBrowser,false);
        tab.removeAttribute("tiletabs-assigned");
        
        if (ptile.childTile.length == 2)  /* exactly 2 children */
        {
            if (stile.type == "panel")  /* 2 panel tiles */
            {
                ptile.type = "panel";
                ptile.panelID = stile.panelID;
                /* Release storage */
                delete tile;
                delete stile;
                ptile.childTile.length = 0;
                ptile.splitterID.length = 0;
                /* Select a tab */
                if (select)
                {
                    tileTabs.prevTiledTab = tileTabs.findTabByPanel(ptile.panelID);
                    gBrowser.selectedTab = tileTabs.prevTiledTab;
                }
                tileTabs.panelCount--;
            }
            else  /* 1 panel tile and 1 hsplit/vsplit tile */
            {
                if (ptile.parentTile == null) /* at root */
                {
                    if (ptile.type == "hsplit")  /* horizontal split */
                    {
                        stile.height = ptile.height;
                        stile.top = ptile.top;
                        stile.ratio = 1;
                        stile.parentTile = null;
                    }
                    else  /* vertical split */
                    {
                        stile.width = ptile.width;
                        stile.left = ptile.left;
                        stile.ratio = 1;
                        stile.parentTile = null;
                    }
                    /* Release storage */
                    delete tile;
                    ptile.childTile.length = 0;
                    ptile.splitterID.length = 0;
                    delete ptile;
                    /* Resize tiles and select a tab */
                    tileTabs.resizeTile(stile,stile.width,stile.height,stile.left,stile.top);
                    for (tile = stile.childTile[0]; tile.type != "panel"; tile = tile.childTile[0]);
                    if (select)
                    {
                        tileTabs.prevTiledTab = tileTabs.findTabByPanel(tile.panelID);
                        gBrowser.selectedTab = tileTabs.prevTiledTab;
                    }
                    tileTabs.rootTile = stile;
                    tileTabs.panelCount--;
                }
                else  /* not at root */
                {
                    num = stile.childTile.length;
                    gptile = ptile.parentTile;
                    for (gpc = 0; gpc < gptile.childTile.length; gpc++) if (gptile.childTile[gpc] == ptile) break;
                    for (i = gptile.childTile.length-1; i > gpc; i--)
                    {
                        gptile.childTile[i+num-1] = gptile.childTile[i];
                        gptile.splitterID[i+num-2] = gptile.splitterID[i-1];
                    }
                    for (i = 0; i < num; i++) gptile.childTile[gpc+i] = stile.childTile[i];
                    for (i = 0; i < num-1; i++) gptile.splitterID[gpc+i] = stile.splitterID[i];
                    if (gptile.type == "hsplit")  /* horizontal split */
                    {
                        for (i = 0; i < num; i++)
                        {
                            gptile.childTile[gpc+i].parentTile = gptile;
                            gptile.childTile[gpc+i].width = gptile.width;
                            gptile.childTile[gpc+i].left = gptile.left;
                        }
                        for (i = 0; i < gptile.childTile.length; i++)
                            gptile.childTile[i].ratio = gptile.childTile[i].height/(gptile.height-gptile.splitterID.length*tileTabs.splitterSize);
                    }
                    else  /* vertical split */
                    {
                        for (i = 0; i < num; i++)
                        {
                            gptile.childTile[gpc+i].parentTile = gptile;
                            gptile.childTile[gpc+i].height = gptile.height;
                            gptile.childTile[gpc+i].top = gptile.top;
                        }
                        for (i = 0; i < gptile.childTile.length; i++)
                            gptile.childTile[i].ratio = gptile.childTile[i].width/(gptile.width-gptile.splitterID.length*tileTabs.splitterSize);
                    }
                    /* Release storage */
                    delete tile;
                    stile.childTile.length = 0;
                    stile.splitterID.length = 0;
                    delete stile;
                    ptile.childTile.length = 0;
                    ptile.splitterID.length = 0;
                    delete ptile;
                    /* Resize expanded tile and select a tab */
                    tileTabs.resizeTile(gptile,gptile.width,gptile.height,gptile.left,gptile.top);
                    for (tile = gptile.childTile[gpc]; tile.type != "panel"; tile = tile.childTile[0]);
                    if (select)
                    {
                        tileTabs.prevTiledTab = tileTabs.findTabByPanel(tile.panelID);
                        gBrowser.selectedTab = tileTabs.prevTiledTab;
                    }
                    tileTabs.panelCount--;
                }
            }
        }
        else  /* more than 2 children */
        {
            if (ptile.type == "hsplit")  /* horizontal split */
            {
                stile.height += tile.height+tileTabs.splitterSize;
                if (pc == 0) stile.top = tile.top;
                for (i = 0; i < ptile.childTile.length; i++) if (i != pc)
                    ptile.childTile[i].ratio = ptile.childTile[i].height/(ptile.height-(ptile.splitterID.length-1)*tileTabs.splitterSize);
            }
            else  /* vertical split */
            {
                stile.width += tile.width+tileTabs.splitterSize;
                if (pc == 0) stile.left = tile.left;
                for (i = 0; i < ptile.childTile.length; i++) if (i != pc)
                    ptile.childTile[i].ratio = ptile.childTile[i].width/(ptile.width-(ptile.splitterID.length-1)*tileTabs.splitterSize);
            }
            for (i = pc; i < ptile.childTile.length-1; i++)
            {
                ptile.childTile[i] = ptile.childTile[i+1];
                if (i > 0) ptile.splitterID[i-1] = ptile.splitterID[i];
            }
            /* Release storage */
            delete tile;
            ptile.childTile.length--;
            ptile.splitterID.length--;
            /* Resize sibling tile and select a tab */
            tileTabs.resizeTile(ptile,ptile.width,ptile.height,ptile.left,ptile.top);
            for (tile = stile; tile.type != "panel"; tile = tile.childTile[0]);
            if (select)
            {
                tileTabs.prevTiledTab = tileTabs.findTabByPanel(tile.panelID);
                gBrowser.selectedTab = tileTabs.prevTiledTab;
            }
            tileTabs.panelCount--;
        }
    },
    
    removeTileLast: function()
    {
        var tab,notificationBox,browser;
        
        /* Remove tile and listeners */
        notificationBox = document.getElementById(tileTabs.rootTile.panelID);
        notificationBox.removeEventListener("click",tileTabs.onClickPanel,false);
        notificationBox.removeEventListener("mousedown",tileTabs.onMouseDownPanel,false);
        tab = tileTabs.findTabByPanel(tileTabs.rootTile.panelID);
        browser = tab.linkedBrowser;
        browser.removeEventListener("scroll",tileTabs.onScrollBrowser,false);
        tab.removeAttribute("tiletabs-assigned");
        /* Release storage */
        delete ptile;
        /* Select a tab */
        tileTabs.rootTile = null;
        tileTabs.panelCount--;
    },
    
    /********************************************************************/
    
    /* Tile resize functions */
    
    resizeTile: function(tile,newWidth,newHeight,newLeft,newTop)
    {
        var i,width,height,left,top;
        
        if (tile.type == "panel")  /* panel tile */
        {
        }
        else  /* split tile */
        {
            if (tile.type == "hsplit")  /* horizontal split */
            {
                if (newHeight >= tile.splitterID.length*tileTabs.splitterSize)
                {
                    top = newTop;
                    for (i = 0; i < tile.childTile.length-1; i++)
                    {
                        height = Math.floor((newHeight-tile.splitterID.length*tileTabs.splitterSize)*tile.childTile[i].ratio);
                        tileTabs.resizeTile(tile.childTile[i],newWidth,height,newLeft,top);
                        top += tile.childTile[i].height+tileTabs.splitterSize;
                    }
                    tileTabs.resizeTile(tile.childTile[i],newWidth,newHeight-(top-newTop),newLeft,top);
                }
                else
                {
                    for (i = 0; i < tile.childTile.length; i++)
                    {
                        tileTabs.resizeTile(tile.childTile[i],newWidth,0,newLeft,newTop);
                    }
                }
            }
            else  /* vertical split */
            {
                if (newWidth >= tile.splitterID.length*tileTabs.splitterSize)
                {
                    left = newLeft;
                    for (i = 0; i < tile.childTile.length-1; i++)
                    {
                        width = Math.floor((newWidth-tile.splitterID.length*tileTabs.splitterSize)*tile.childTile[i].ratio);
                        tileTabs.resizeTile(tile.childTile[i],width,newHeight,left,newTop);
                        left += tile.childTile[i].width+tileTabs.splitterSize;
                    }
                    tileTabs.resizeTile(tile.childTile[i],newWidth-(left-newLeft),newHeight,left,newTop);
                }
                else
                {
                    for (i = 0; i < tile.childTile.length; i++)
                    {
                        tileTabs.resizeTile(tile.childTile[i],0,newHeight,newLeft,newTop);
                    }
                }
            }
        }
        
        tile.width = newWidth;
        tile.height = newHeight;
        tile.left = newLeft;
        tile.top = newTop;
    },
    
    equalizeTile: function(tile)
    {
        var i,width,height,left,top;
        
        if (tile.type == "hsplit")  /* horizontal split */
        {
            height = Math.floor((tile.height-tile.splitterID.length*tileTabs.splitterSize)/tile.childTile.length);
            top = tile.top;
            for (i = 0; i < tile.childTile.length-1; i++)
            {
                tileTabs.resizeTile(tile.childTile[i],tile.childTile[i].width,height,tile.childTile[i].left,top);
                tile.childTile[i].ratio = 1/tile.childTile.length;
                top += height+tileTabs.splitterSize;
            }
            tileTabs.resizeTile(tile.childTile[i],tile.childTile[i].width,tile.height-(top-tile.top),tile.childTile[i].left,top);
            tile.childTile[i].ratio = 1/tile.childTile.length;
        }
        else  /* vertical split */
        {
            width = Math.floor((tile.width-tile.splitterID.length*tileTabs.splitterSize)/tile.childTile.length);
            left = tile.left;
            for (i = 0; i < tile.childTile.length-1; i++)
            {
                tileTabs.resizeTile(tile.childTile[i],width,tile.childTile[i].height,left,tile.childTile[i].top);
                tile.childTile[i].ratio = 1/tile.childTile.length;
                left += width+tileTabs.splitterSize;
            }
            tileTabs.resizeTile(tile.childTile[i],tile.width-(left-tile.left),tile.childTile[i].height,left,tile.childTile[i].top);
            tile.childTile[i].ratio = 1/tile.childTile.length;
        }
    },
    
    /********************************************************************/
    
    /* Tile display functions */
    
    drawTile: function(tile)
    {
        var i,left,top,tab,notificationBox,splitterBox;
        
        if (tile.type == "panel")  /* panel tile */
        {
            notificationBox = document.getElementById(tile.panelID);
            if (notificationBox != null)  /* in case browser has been closed without tile being removed */
            {
                notificationBox.setAttribute("style","margin-left: " + tile.left + "px;" +
                                                     "margin-top: " + tile.top + "px;" +
                                                     "margin-right: " + (tileTabs.tabPanels.boxObject.width-(tile.left+tile.width)) + "px;" +
                                                     "margin-bottom: " + (tileTabs.tabPanels.boxObject.height-(tile.top+tile.height)) + "px;" +
                                                     "border: " + tileTabs.tileBorder + "px solid #FFFFFF;");
                
                if (tileTabs.ffVersion >= "4.0")
                {
                    tab = tileTabs.findTabByPanel(notificationBox.id);
                    for (i = 0; i < gBrowser.visibleTabs.length; i++) if (tab == gBrowser.visibleTabs[i]) break;
                    if (i == gBrowser.visibleTabs.length)
                    {
                        notificationBox.setAttribute("style","margin-left: " + tile.left + "px;" +
                                                             "margin-top: " + tile.top + "px;" +
                                                             "margin-right: " + (tileTabs.tabPanels.boxObject.width-(tile.left+tile.width)) + "px;" +
                                                             "margin-bottom: " + (tileTabs.tabPanels.boxObject.height-(tile.top+tile.height)) + "px;" +
                                                             "border: " + tileTabs.tileBorder + "px dotted " + tileTabs.skinColor + ";");
                    }
                }
            }
        }
        else  /* split tile */
        {
            if (tile.type == "hsplit")  /* horizontal split */
            {
                if (tile.height >= tile.splitterID.length*tileTabs.splitterSize)
                {
                    top = tile.top-tileTabs.splitterSize;
                    for (i = 0; i < tile.splitterID.length; i++)
                    {
                        top += tile.childTile[i].height+tileTabs.splitterSize;
                        splitterBox = document.getElementById(tile.splitterID[i]);
                        splitterBox.setAttribute("style","margin-left: " + tile.left + "px;" +
                                                         "margin-top: " + top + "px;" + 
                                                         "margin-right: " + (tileTabs.tabPanels.boxObject.width-(tile.left+tile.width)) + "px;" +
                                                         "margin-bottom: " + (tileTabs.tabPanels.boxObject.height-(top+tileTabs.splitterSize)) + "px;" +
                                                         "border-top: 1px solid #A0A0A0;" +
                                                         "border-bottom:1px solid #A0A0A0;" +
                                                         "background: #F0F0F0; cursor: n-resize;");
                    }
                }
                else
                {
                    for (i = 0; i < tile.splitterID.length; i++)
                    {
                        splitterBox = document.getElementById(tile.splitterID[i]);
                        splitterBox.setAttribute("style","visibility: hidden;");
                    }
                }
            }
            else  /* vertical split */
            {
                if (tile.width >= tile.splitterID.length*tileTabs.splitterSize)
                {
                    left = tile.left-tileTabs.splitterSize;
                    for (i = 0; i < tile.splitterID.length; i++)
                    {
                        left += tile.childTile[i].width+tileTabs.splitterSize;
                        splitterBox = document.getElementById(tile.splitterID[i]);
                        splitterBox.setAttribute("style","margin-left: " + left + "px;" +
                                                         "margin-top: " + tile.top + "px;" +
                                                         "margin-right: " + (tileTabs.tabPanels.boxObject.width-(left+tileTabs.splitterSize)) + "px;" +
                                                         "margin-bottom: " + (tileTabs.tabPanels.boxObject.height-(tile.top+tile.height)) + "px;" +
                                                         "border-left: 1px solid #A0A0A0;" +
                                                         "border-right:1px solid #A0A0A0;" +
                                                         "background: #F0F0F0; cursor: e-resize;");
                    }
                }
                else
                {
                    for (i = 0; i < tile.splitterID.length; i++)
                    {
                        splitterBox = document.getElementById(tile.splitterID[i]);
                        splitterBox.setAttribute("style","visibility: hidden;");
                    }
                }
            }
            
            for (i = 0; i < tile.childTile.length; i++)
                tileTabs.drawTile(tile.childTile[i]);
        }
    },

    highlightPanel: function()
    {
        var tile,notificationBox;
        
        tile = tileTabs.findTileByPanel(tileTabs.rootTile,gBrowser.selectedTab.linkedPanel);
        notificationBox = document.getElementById(gBrowser.selectedTab.linkedPanel);
        if (notificationBox != null)  /* in case browser has been closed without tile being removed */
        {
            notificationBox.setAttribute("style","margin-left: " + tile.left + "px;" +
                                                 "margin-top: " + tile.top + "px;" +
                                                 "margin-right: " + (tileTabs.tabPanels.boxObject.width-(tile.left+tile.width)) + "px;" +
                                                 "margin-bottom: " + (tileTabs.tabPanels.boxObject.height-(tile.top+tile.height)) + "px;" +
                                                 "border: " + tileTabs.tileBorder + "px solid " + tileTabs.skinColor + ";");
        }
        
        if (gBrowser.selectedTab.linkedBrowser.currentURI.spec == "about:addons")
            document.getElementById("main-window").removeAttribute("disablechrome");
    },
    
    highlightGroup: function(tile,active)
    {
        var i,panel,tab;
        
        if (tile.type == "panel")
        {
            panel = document.getElementById(tile.panelID);
            if (active)
            {
                panel.style.setProperty("border-style","dashed","");
                panel.style.setProperty("border-color",tileTabs.skinColor,"");
            }            
            else
            {
                panel.style.setProperty("border-style","solid","");
                panel.style.setProperty("border-color","#FFFFFF","");
                if (tile.panelID == gBrowser.selectedTab.linkedPanel) panel.style.setProperty("border-color",tileTabs.skinColor,"");
                if (tileTabs.ffVersion >= "4.0")
                {
                    tab = tileTabs.findTabByPanel(tile.panelID);
                    for (i = 0; i < gBrowser.visibleTabs.length; i++) if (tab == gBrowser.visibleTabs[i]) break;
                    if (i == gBrowser.visibleTabs.length)
                    {
                        panel.style.setProperty("border-style","dotted","");
                        panel.style.setProperty("border-color",tileTabs.skinColor,"");
                    }
                }
            }
        }
        else for (i = 0; i < tile.childTile.length; i++) tileTabs.highlightGroup(tile.childTile[i],active);
    },
    
    hideNonTiledPanels: function()
    {
        var i,tab,notificationBox;
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            tab = gBrowser.tabContainer.childNodes[i];
            if (tab.getAttribute("tiletabs-assigned") != "true")
            {
                notificationBox = document.getElementById(tab.linkedPanel);
                notificationBox.setAttribute("style","visibility: hidden;");
                
                /* docShell.isActive enables/disables browser display updating in Firefox 4.0 */
                if (tileTabs.ffVersion >= "4.0") tab.linkedBrowser.docShell.isActive = false;
            }
            else 
            {
                /* docShell.isActive enables/disables browser display updating in Firefox 4.0 */
                if (tileTabs.ffVersion >= "4.0") tab.linkedBrowser.docShell.isActive = true;
            }
        }
    },
    
    /********************************************************************/
    
    /* Normal display functions */
        
    drawSelectedPanel: function()
    {
        var notificationBox;
        
        notificationBox = document.getElementById(gBrowser.selectedTab.linkedPanel);
        notificationBox.setAttribute("style","");
        
        if (gBrowser.selectedTab.linkedBrowser.currentURI.spec == "about:addons")
            document.getElementById("main-window").setAttribute("disablechrome","true");
    },
    
    hideNonSelectedPanels: function()
    {
        var i,child,tab,notificationBox,splitterBox;
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            tab = gBrowser.tabContainer.childNodes[i];
            if (tab != gBrowser.selectedTab)
            {
                notificationBox = document.getElementById(tab.linkedPanel);
                notificationBox.setAttribute("style","visibility: hidden;");
                
                /* docShell.isActive enables/disables browser display updating in Firefox 4.0 */
                if (tileTabs.ffVersion >= "4.0") tab.linkedBrowser.docShell.isActive = false;
            }
            else
            {
                /* docShell.isActive enables/disables browser display updating in Firefox 4.0 */
                if (tileTabs.ffVersion >= "4.0") tab.linkedBrowser.docShell.isActive = true;
            }
        }
        
        for (i = 0; i < tileTabs.extraBox.childNodes.length; i++)
        {
            child = tileTabs.extraBox.childNodes[i];
            if (child.id.substr(0,17) == "tiletabs-splitter")
            {
                splitterBox = document.getElementById(child.id);
                splitterBox.setAttribute("style","visibility: hidden;");
            }
        }
    },
    
    /********************************************************************/
    
    /* Tile find functions */
    
    findTileByPanel: function(tile,panelID)
    {
        var i,value;
        
        if (tile.type == "panel")  /* panel tile */
        {
            if (tile.panelID == panelID) return tile;
        }
        else  /* split tile */
        {
            for (i = 0; i < tile.childTile.length; i++)
                if ((value = tileTabs.findTileByPanel(tile.childTile[i],panelID)) != null) return value;
        }
        return null;
    },
    
    findTileBySplitter: function(tile,splitterID)
    {
        var i,value;
        
        if (tile.type == "panel")  /* panel tile */
        {
        }
        else  /* split tile */
        {
            for (i = 0; i < tile.splitterID.length; i++)
                if (tile.splitterID[i] == splitterID) return tile;
            
            for (i = 0; i < tile.childTile.length; i++)
                if ((value = tileTabs.findTileBySplitter(tile.childTile[i],splitterID)) != null) return value;
        }
        return null;
    },
    
    /********************************************************************/
    
    /* Tab find function */
    
    findTabByPanel: function(panelID)
    {
        var i,tab;
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            tab = gBrowser.tabContainer.childNodes[i];
            if (tab.linkedPanel == panelID) return tab;
        }
        
        return null;
    },
    
    /********************************************************************/
    
    /* Tiled Tabs style function */
    
    styleTiledTabs: function()
    {
        var i,tab,label,icon;
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            tab = gBrowser.tabContainer.childNodes[i];
            if (tileTabs.ffVersion == "3.6") label = document.getAnonymousElementByAttribute(tab,"class","tab-text");
            else if (tileTabs.ffVersion >= "4.0") label = document.getAnonymousElementByAttribute(tab,"class","tab-text tab-label");
            icon = document.getAnonymousElementByAttribute(tab,"class","tab-icon-image");
            if (tab.getAttribute("tiletabs-assigned") == "true")
            {
                if (tileTabs.tabSingle && tab != tileTabs.prevTiledTab && tileTabs.viewTiled) tab.style.setProperty("display","none","");
                else tab.style.removeProperty("display");
                if (tileTabs.prefs.getBoolPref("tab-iconborder"))
                {
                    icon.style.setProperty("border","2px solid " + tileTabs.skinColor,"");
                }
                else
                {
                    icon.style.removeProperty("width");
                    icon.style.removeProperty("height");
                    icon.style.removeProperty("margin");
                    icon.style.removeProperty("border");
                }
                if (tileTabs.prefs.getBoolPref("tab-colortext")) label.style.setProperty("color",tileTabs.skinColor,"");
                else label.style.removeProperty("color");
                if (tileTabs.prefs.getBoolPref("tab-italictext")) label.style.setProperty("font-style","italic","");
                else label.style.removeProperty("font-style");
            }
            else
            {
                tab.style.removeProperty("display");
                icon.style.removeProperty("width");
                icon.style.removeProperty("height");
                icon.style.removeProperty("margin");
                icon.style.removeProperty("border");
                label.style.removeProperty("color");
                label.style.removeProperty("font-style");
            }
        }
    },
    
    /********************************************************************/
    
    /* Tiled Tabs move function */
    
    moveTiledTabs: function()
    {
        var i,j,tab;
        
        if (tileTabs.tabMove)
        {
            j = 0;        
            for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
            {
                tab = gBrowser.tabContainer.childNodes[i];
                if (tab.getAttribute("tiletabs-assigned") == "true") gBrowser.moveTabTo(tab,j++);
            }
        }
    },
    
    /********************************************************************/
    
    /* Tab assign and unassign functions */
    
    assignTab: function()
    {
        var i,j,tab;
        
        if (tileTabs.prefs.getBoolPref("tab-assignexist"))
        {
            if (tileTabs.prefs.getBoolPref("tab-assignleft"))  /* assign tab from left if possible else from right */
            {
                for (i = gBrowser.selectedTab._tPos-1; i >= 0; i--)
                {
                    tab = gBrowser.tabContainer.childNodes[i];
                    if (tab.getAttribute("tiletabs-assigned") != "true")
                    {
                        if (tileTabs.ffVersion >= "4.0")
                        {
                            for (j = 0; j < gBrowser.visibleTabs.length; j++)
                                if (gBrowser.visibleTabs[j] == tab) return tab;
                        }
                        else return tab;
                    }
                }
                
                for (i = gBrowser.selectedTab._tPos+1; i < gBrowser.tabContainer.childNodes.length; i++)
                {
                    tab = gBrowser.tabContainer.childNodes[i];
                    if (tab.getAttribute("tiletabs-assigned") != "true")
                    {
                        if (tileTabs.ffVersion >= "4.0")
                        {
                            for (j = 0; j < gBrowser.visibleTabs.length; j++)
                                if (gBrowser.visibleTabs[j] == tab) return tab;
                        }
                        else return tab;
                    }
                }
            }
            else  /* assign tab from right if possible else from left */
            {
                for (i = gBrowser.selectedTab._tPos+1; i < gBrowser.tabContainer.childNodes.length; i++)
                {
                    tab = gBrowser.tabContainer.childNodes[i];
                    if (tab.getAttribute("tiletabs-assigned") != "true")
                    {
                        if (tileTabs.ffVersion >= "4.0")
                        {
                            for (j = 0; j < gBrowser.visibleTabs.length; j++)
                                if (gBrowser.visibleTabs[j] == tab) return tab;
                        }
                        else return tab;
                    }
                }
                
                for (i = gBrowser.selectedTab._tPos-1; i >= 0; i--)
                {
                    tab = gBrowser.tabContainer.childNodes[i];
                    if (tab.getAttribute("tiletabs-assigned") != "true")
                    {
                        if (tileTabs.ffVersion >= "4.0")
                        {
                            for (j = 0; j < gBrowser.visibleTabs.length; j++)
                                if (gBrowser.visibleTabs[j] == tab) return tab;
                        }
                        else return tab;
                    }
                }
            }
        }
        
        if (tileTabs.prefs.getBoolPref("tab-loadhome")) tab = gBrowser.loadOneTab(gHomeButton.getHomePage().split("|",1),null,null,null,true,false);
        else tab = gBrowser.loadOneTab("about:blank",null,null,null,true,false);
        
        tab.setAttribute("tiletabs-generated","true"); 
        
        return tab;
    },
    
    unassignTab: function(tab)
    {
        var currentURI,homePage;
        
        if (tab.getAttribute("tiletabs-generated") == "true")
        {
            tab.removeAttribute("tiletabs-generated"); 
            
            if (tab != gBrowser.selectedTab)  /* see Note 3 - consider changing in future */
            {
                currentURI = tab.linkedBrowser.currentURI.spec;
                if (currentURI.substr(currentURI.length-1) == "/") currentURI = currentURI.substr(0,currentURI.length-1);
                
                homePage = gHomeButton.getHomePage().split("|",1)[0];
                if (homePage.substr(homePage.length-1) == "/") homePage = homePage.substr(0,homePage.length-1);
                
                if (currentURI == "about:blank" || currentURI == homePage)
                {
                    gBrowser.removeTab(tab);  /* ignored if last tab */  /* generates a TabClose event */
                    return true;
                }
            }
        }
       
        return false;
    },
    
    /********************************************************************/
    
    /* Detector event function */
    
    onDetectorAreaChanged: function(event)
    {
        if (tileTabs.tabPanels.boxObject.width == 0 || tileTabs.tabPanels.boxObject.height == 0) return;  /* happens in print preview when changing portrait/landcape */
        
        if (tileTabs.panelCount > 0) tileTabs.resizeTile(tileTabs.rootTile,tileTabs.tabPanels.boxObject.width,tileTabs.tabPanels.boxObject.height,0,0);
        
        if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
        {
            tileTabs.hideNonTiledPanels();  /* changed size */
            tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
            tileTabs.highlightPanel();
        }
        else
        {
            tileTabs.hideNonSelectedPanels();  /* changed size */
            tileTabs.drawSelectedPanel();
        }
    },
    
    /********************************************************************/
    
    /* Tab event functions */
    
    onTabOpen: function(event)
    {
        var tab,browser;
        
        tab = event.target;
        
        tab.addEventListener("mousedown",tileTabs.onMouseDownTab,true);
        tab.addEventListener("click",tileTabs.onClickTab,true);
        
        browser = tab.linkedBrowser;
        
        browser.addEventListener("dragstart",tileTabs.onDragStartBrowser,true);
        browser.addEventListener("dragover",tileTabs.onDragOverBrowser,true);
        browser.addEventListener("drop",tileTabs.onDropBrowser,true);
        
        if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
        {
            tileTabs.hideNonTiledPanels();
            tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
            tileTabs.highlightPanel();
        }
        else
        {
            tileTabs.hideNonSelectedPanels();
            tileTabs.drawSelectedPanel();
        }
        tileTabs.styleTiledTabs();
    },
    
    onTabSelect: function(event)
    {
        var tile,srctile,notificationBox,browser;
        
        if (tileTabs.viewTiled)
        {
            if (tileTabs.prefs.getBoolPref("tab-assignselect"))
            {
                /* Assign selected tab to active tile */
                
                if (gBrowser.selectedTab != tileTabs.prevTiledTab)
                {
                    tile = tileTabs.findTileByPanel(tileTabs.rootTile,tileTabs.prevTiledTab.linkedPanel);
                    
                    if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true")
                    {
                        srctile = tileTabs.findTileByPanel(tileTabs.rootTile,gBrowser.selectedTab.linkedPanel);
                        
                        /* Attach selected tab to active tile */
                        
                        tile.panelID = gBrowser.selectedTab.linkedPanel;
                        
                        /* Attach previous tab to source tile */
                        
                        srctile.panelID = tileTabs.prevTiledTab.linkedPanel;
                    }
                    else
                    {
                        /* Detach previous tab from active tile */
                        
                        notificationBox = document.getElementById(tileTabs.prevTiledTab.linkedPanel);
                        notificationBox.removeEventListener("click",tileTabs.onClickPanel,false);
                        notificationBox.removeEventListener("mousedown",tileTabs.onMouseDownPanel,false);
                        
                        browser = tileTabs.prevTiledTab.linkedBrowser;
                        browser.removeEventListener("scroll",tileTabs.onScrollBrowser,false);
                        
                        tileTabs.prevTiledTab.removeAttribute("tiletabs-assigned");
                        
                        tileTabs.unassignTab(tileTabs.prevTiledTab);
                        
                        /* Attach selected tab to active tile */
                        
                        tile.panelID = gBrowser.selectedTab.linkedPanel;
                        
                        notificationBox = document.getElementById(gBrowser.selectedTab.linkedPanel);
                        notificationBox.addEventListener("click",tileTabs.onClickPanel,false);
                        notificationBox.addEventListener("mousedown",tileTabs.onMouseDownPanel,false);
                        
                        browser = gBrowser.selectedTab.linkedBrowser;
                        browser.addEventListener("scroll",tileTabs.onScrollBrowser,false);
                        
                        gBrowser.selectedTab.setAttribute("tiletabs-assigned","true");
                    }
                    
                    tileTabs.saveRestoreLayout();
                }
            }
        }
        
        if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true") tileTabs.prevTiledTab = gBrowser.selectedTab;
        
        if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
        {
            tileTabs.hideNonTiledPanels();
            tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
            tileTabs.highlightPanel();
        }
        else
        {
            tileTabs.hideNonSelectedPanels();
            tileTabs.drawSelectedPanel();
        }
        tileTabs.styleTiledTabs();
        tileTabs.moveTiledTabs();
    },
    
    onTabClose: function(event)
    {
        var closedTab,tile,newTab,notificationBox,browser,tab,buttonmenu;
        
        closedTab = event.originalTarget;
        
        if (closedTab.getAttribute("tiletabs-assigned") == "true")
        {
            tile = tileTabs.findTileByPanel(tileTabs.rootTile,closedTab.linkedPanel);
            
            if (tileTabs.prefs.getBoolPref("tab-assignclose"))
            {
                /* Assign another tab to tile when tiled tab closed */
                
                newTab = tileTabs.assignTab();
                
                tile.panelID = newTab.linkedPanel;
                
                notificationBox = document.getElementById(newTab.linkedPanel);
                notificationBox.addEventListener("click",tileTabs.onClickPanel,false);
                notificationBox.addEventListener("mousedown",tileTabs.onMouseDownPanel,false);
                
                browser = newTab.linkedBrowser;
                browser.addEventListener("scroll",tileTabs.onScrollBrowser,false);
                
                newTab.setAttribute("tiletabs-assigned","true");
                
                if (closedTab == tileTabs.prevTiledTab)
                {
                    tileTabs.prevTiledTab = newTab;
                    gBrowser.selectedTab = tileTabs.prevTiledTab;
                }
                
                tileTabs.saveRestoreLayout();
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                {
                    tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                    tileTabs.highlightPanel();
                }
            }
            else  /* remove  tile */
            {
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                {
                    if (closedTab == gBrowser.selectedTab) tileTabs.removeTile(tile,true);
                    else tileTabs.removeTile(tile,false);
                }
                else tileTabs.removeTile(tile,false);
                
                if (tileTabs.panelCount > 1)
                {       
                    tileTabs.saveRestoreLayout();
                    
                    if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                    {
                        tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                        tileTabs.highlightPanel();
                    }
                }
                else  /* exit tile mode */
                {
                    tab = tileTabs.findTabByPanel(tileTabs.rootTile.panelID);
                    tileTabs.removeTileLast();
                    tileTabs.unassignTab(tab);
                    
                    tileTabs.deleteRestoreLayout();
                    
                    tileTabs.syncScroll = false;
                    tileTabs.viewTiled = false;
                    
                    if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
                    {
                        buttonmenu.removeAttribute("layout");
                        document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").removeAttribute("checked");
                    }
                    
                    tileTabs.hideNonSelectedPanels();
                    tileTabs.drawSelectedPanel();
                }
            }
            tileTabs.styleTiledTabs();
            tileTabs.moveTiledTabs();
        }
    },
    
    onTabMove: function(event)
    {
        var tab;
        
        if (tileTabs.tabMove)
        {
            tab = event.target;
            
            if (tab.getAttribute("tiletabs-assigned") == "true")
            {
                if (tab._tPos >= tileTabs.panelCount) gBrowser.moveTabTo(tab,tileTabs.panelCount-1);
            }
            else
            {
                if (tab._tPos < tileTabs.panelCount) gBrowser.moveTabTo(tab,tileTabs.panelCount);
            }
        }
        tileTabs.styleTiledTabs();
        
        tileTabs.saveRestoreLayout();  /* tab positions have changed */
    },
    
    onTabShowHide: function(event)
    {
        if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
        {
            tileTabs.hideNonTiledPanels();
            tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
            tileTabs.highlightPanel();
        }
        else
        {
            tileTabs.hideNonSelectedPanels();
            tileTabs.drawSelectedPanel();
        }
        tileTabs.styleTiledTabs();
        
        tileTabs.saveRestoreLayout();  /* tab positions have changed */
    },
    
    onMouseDownTab: function(event)
    {
        /* postpone tab select - necessary to allow drag-and-drop tiling/copying of untiled tabs */
        
        if (event.button == 0)
        {
            modifiers = event.altKey*4+(event.ctrlKey||event.metaKey)*2+event.shiftKey;
            
            if (tileTabs.mouseTile != 0 && tileTabs.mouseTile == modifiers) event.stopPropagation();  /* drag tiling of tab */
            else if ((tileTabs.osPlatform != "Darwin" && event.ctrlKey) || (tileTabs.osPlatform == "Darwin" && event.altKey)) event.stopPropagation();  /* drag copying of tab */
        }
    },
    
    onClickTab: function(event)
    {
        var modifiers;
        
        if (event.originalTarget.localName != "toolbarbutton")  /* not click on tab 'x' close button */
        {    
            /* reinstate tab select - necessary to allow drag-and-drop tiling/copying of untiled tabs */
            
            if (event.button == 0)
            {
                modifiers = event.altKey*4+(event.ctrlKey||event.metaKey)*2+event.shiftKey;
                
                if (tileTabs.mouseTile != 0 && tileTabs.mouseTile == modifiers)  /* drag tiling of tab */
                {
                    setTimeout(function(tab) { gBrowser.selectedTab = tab },1,event.target);  /* see Note B */
                }
                else if ((tileTabs.osPlatform != "Darwin" && event.ctrlKey) || (tileTabs.osPlatform == "Darwin" && event.altKey))  /* drag copying of tab */
                {
                    setTimeout(function(tab) { gBrowser.selectedTab = tab },1,event.target);  /* see Note B */
                }
            }
            
            /* click assign tab to tile */
            
            if (!tileTabs.prefs.getBoolPref("tab-assignselect"))
            {
                if (event.button == 0)
                {
                    if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                    {
                        modifiers = event.altKey*4+(event.ctrlKey||event.metaKey)*2+event.shiftKey;
                        
                        if (tileTabs.mouseAssign != 0 && tileTabs.mouseAssign == modifiers)  /* click replacing */
                        {
                            tileTabs.menuActions('assign',event.target._tPos);
                        }
                    }
                }
            }
        }
    },
    
    /********************************************************************/
    
    /* Panel event function */
    
    onMouseDownPanel: function(event)
    {
        if (event.button != 2) return;  /* see Note C */
        
        if (document.getElementById("print-preview-toolbar") != null) return;  /* print preview mode */
        
        tileTabs.prevTiledTab = tileTabs.findTabByPanel(event.currentTarget.id);
        gBrowser.selectedTab = tileTabs.prevTiledTab;
    },
    
    onClickPanel: function(event)
    {
        if (event.button != 0) return;  /* see Note D */
        
        if (document.getElementById("print-preview-toolbar") != null) return;  /* print preview mode */
        
        tileTabs.prevTiledTab = tileTabs.findTabByPanel(event.currentTarget.id);
        gBrowser.selectedTab = tileTabs.prevTiledTab;
    },
    
    /********************************************************************/
    
    /* Browser event functions */
    
    onScrollBrowser: function(event)
    {
        var i,browser,tab,tile,notificationBox,scrollX,scrollY;
        
        browser = event.currentTarget;
        notificationBox = gBrowser.getNotificationBox(browser);
        tab = tileTabs.findTabByPanel(notificationBox.id)    
        tile = tileTabs.findTileByPanel(tileTabs.rootTile,notificationBox.id);
        
        scrollX = browser.contentWindow.scrollX;
        scrollY = browser.contentWindow.scrollY;
        
        if (tileTabs.syncScroll && (!tileTabs.prefs.getBoolPref("tile-syncactive") || tab == gBrowser.selectedTab))
        {
            for (i = 0; i < tile.parentTile.childTile.length; i++)
            {
                if (tile.parentTile.childTile[i] != tile && tile.parentTile.childTile[i].type == "panel")
                {
                    notificationBox = document.getElementById(tile.parentTile.childTile[i].panelID);
                    tab = tileTabs.findTabByPanel(tile.parentTile.childTile[i].panelID)    
                    browser = tab.linkedBrowser;
                    browser.contentWindow.scrollTo(scrollX+(tile.parentTile.childTile[i].scrollX-tile.scrollX),scrollY+(tile.parentTile.childTile[i].scrollY-tile.scrollY));
                    
                    tile.parentTile.childTile[i].scrollX = browser.contentWindow.scrollX;
                    tile.parentTile.childTile[i].scrollY = browser.contentWindow.scrollY;
                }
            }
        }
        
        tile.scrollX = scrollX;
        tile.scrollY = scrollY;
    },
    
    onDragStartBrowser: function(event)
    {
        /* necessary to detect if drag within same tile */
        
        if (tileTabs.dragBrowser == null)  /* drag start in source panel - not when moving over intermediate panels */
        {
            tileTabs.dragBrowser = event.currentTarget;
        }
    },
    
    onDragOverBrowser: function(event)
    {
        var tab,browser,modifiers,menuPopup,aspectRatio,remainderY,direction,menuItem;
        
        tab = event.dataTransfer.mozGetDataAt("application/x-moz-tabbrowser-tab",0);
        
        browser = event.currentTarget;
        
        modifiers = event.altKey*4+(event.ctrlKey||event.metaKey)*2+event.shiftKey;
        
        if (tileTabs.mouseTile != 0 && tileTabs.mouseTile == modifiers)  /* drag tiling of tab or link */
        {
            if (tileTabs.dragTimeout != null) window.clearTimeout(tileTabs.dragTimeout);
            tileTabs.dragTimeout = window.setTimeout(tileTabs.onDragLeaveBrowserDelayed,100);
            
            menuPopup = document.getElementById("tiletabs-dragmenu-popup");
            if (menuPopup.state == "closed") menuPopup.openPopupAtScreen(event.screenX-100,event.screenY+20,false);
            else if (menuPopup.state == "open") menuPopup.moveTo(event.screenX-100,event.screenY+20);
            
            if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" || tileTabs.panelCount == 0)
            {
                aspectRatio = browser.boxObject.width/browser.boxObject.height;
                remainderY = browser.boxObject.height-1-event.clientY;
                
                if (event.clientX >= event.clientY*aspectRatio && event.clientX < remainderY*aspectRatio) direction = "above";
                else if (event.clientX < event.clientY*aspectRatio && event.clientX < remainderY*aspectRatio) direction = "left";
                else if (event.clientX >= event.clientY*aspectRatio && event.clientX >= remainderY*aspectRatio) direction = "right";
                else if (event.clientX < event.clientY*aspectRatio && event.clientX >= remainderY*aspectRatio) direction = "below";
                
                menuItem = document.getElementById("tiletabs-dragmenu-direction");
                menuItem.setAttribute("label",document.getElementById("tiletabs-mainmenu-tile-" + direction).getAttribute("label"));
                menuItem.setAttribute("image","chrome://tiletabs/skin/tiletabs-tile-" + direction + ".png");
            }
            else
            {
                menuItem = document.getElementById("tiletabs-dragmenu-direction");
                menuItem.setAttribute("label",document.getElementById("tiletabs-dragmenu-invalid").getAttribute("value"));
                menuItem.setAttribute("image","chrome://tiletabs/skin/tiletabs-drag-invalid.png");
            }
            
            event.preventDefault();  /* allow drag tiling within same panel */  /* prevent tab detaching */
        }
        else if ((tileTabs.osPlatform != "Darwin" && event.ctrlKey) || (tileTabs.osPlatform == "Darwin" && event.altKey))  /* drag copying of tab */
        {
            if (tab != null) event.preventDefault();  /* prevent tab detaching */
        }
    },
    
    onDragLeaveBrowserDelayed: function()
    {
        var menuPopup;
        
        menuPopup = document.getElementById("tiletabs-dragmenu-popup");
        menuPopup.hidePopup();
        tileTabs.dragTimeout = null;
    },
    
    onDropBrowser: function(event)
    {
        var tab,link,uri,browser,modifiers,notificationBox,tile,aspectRatio,remainderY,direction,buttonmenu;
        var postData = { };
        
        tab = event.dataTransfer.mozGetDataAt("application/x-moz-tabbrowser-tab",0);
        
        if (tab == null) link = ""; else link = tab.linkedBrowser.currentURI.spec;  /* Mozilla tabbrowser tab */
        if (link == "") link = event.dataTransfer.getData("text/x-moz-url").split("\n",1)[0];  /* Mozilla specific link */
        if (link == "") link = event.dataTransfer.getData("text/uri-list").split("\n",1)[0];  /* normal link */
        if (link == "") link = event.dataTransfer.getData("text/plain").split("\n",1)[0];  /* selected text */
        
        link = link.replace(/^\s*|\s*$/g,"");
        if (link == "" || link.indexOf(" ",0) != -1 || /^\s*(javascript|data):/.test(link)) { tileTabs.dragBrowser = null; return; }
        
        if (link.substr(0,6) != "about:")
        {
            try { nsDragAndDrop.dragDropSecurityCheck(event,null,link); }
            catch (e) { tileTabs.dragBrowser = null; return; }
        }
        
        uri = getShortcutOrURI(link,postData);
        if (!uri) { tileTabs.dragBrowser = null; return; }
        
        browser = event.currentTarget;
        
        modifiers = event.altKey*4+(event.ctrlKey||event.metaKey)*2+event.shiftKey;
        
        if (tileTabs.mouseTile != 0 && tileTabs.mouseTile == modifiers)  /* drag tiling of tab or link - open new tile using new tab (unless dragging untiled tab) */
        {
            if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" || tileTabs.panelCount == 0)
            {
                aspectRatio = browser.boxObject.width/browser.boxObject.height;
                remainderY = browser.boxObject.height-1-event.clientY;
                
                if (event.clientX >= event.clientY*aspectRatio && event.clientX < remainderY*aspectRatio) direction = "above";
                else if (event.clientX < event.clientY*aspectRatio && event.clientX < remainderY*aspectRatio) direction = "left";
                else if (event.clientX >= event.clientY*aspectRatio && event.clientX >= remainderY*aspectRatio) direction = "right";
                else if (event.clientX < event.clientY*aspectRatio && event.clientX >= remainderY*aspectRatio) direction = "below";
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true")
                {
                    notificationBox = gBrowser.getNotificationBox(browser);
                    tile = tileTabs.findTileByPanel(tileTabs.rootTile,notificationBox.id);
                }
                else
                {
                    tileTabs.addTileFirst(gBrowser.selectedTab);
                    tileTabs.prevTiledTab = gBrowser.selectedTab;
                    tile = tileTabs.rootTile;
                    
                    if (tileTabs.tabMove) gBrowser.moveTabTo(gBrowser.selectedTab,tileTabs.panelCount-1);
                    
                    tileTabs.viewTiled = true;
                    
                    if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
                    {
                        buttonmenu.setAttribute("layout","true");
                        document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").setAttribute("checked","true");
                    }
                }
                
                if (tab == null || tab.parentNode != gBrowser.tabContainer || tab.getAttribute("tiletabs-assigned") == "true")
                    tab = gBrowser.loadOneTab(uri,null,null,postData.value,true,false);  /* new tab not required for untiled tab */
                    /* this load changes boxObject.width & boxObject.height and event.clientX & event.clientY ! */
                
                tileTabs.addTile(tile,tab,direction);
                
                if (tileTabs.tabMove) gBrowser.moveTabTo(tab,tileTabs.panelCount-1);
                
                tileTabs.saveRestoreLayout();
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                {
                    tileTabs.hideNonTiledPanels();
                    tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                    tileTabs.highlightPanel();
                }
                else
                {
                    tileTabs.hideNonSelectedPanels();
                    tileTabs.drawSelectedPanel();
                }
                tileTabs.styleTiledTabs();
            }
            
            event.stopPropagation();  /* stop any other drop event handling */
            
            event.preventDefault();
        }
        else  /* drag copying of tab or link - open in existing tile using existing tab  */
        {
            notificationBox = gBrowser.getNotificationBox(browser);
            tab = tileTabs.findTabByPanel(notificationBox.id);
            if (tab == gBrowser.selectedTab)
            {
                if (tileTabs.ffVersion == "3.6") browser.loadURIWithFlags(uri,Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE,null,null,postData.value);
                else ;  /* normal default drag action - load page in selected tab */
            }
            else
            {
                if (event.explicitOriginalTarget.localName == "input" || event.explicitOriginalTarget.localName == "textarea");  /* normal default drag action - paste as text */
                else
                {
                    if (browser == tileTabs.dragBrowser);  /* normal default drag action - no drag within same page  */
                    else
                    {
                        browser.loadURIWithFlags(uri,Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE,null,null,postData.value);
                        
                        event.preventDefault();
                    }
                }
            }
        }
        
        tileTabs.dragBrowser = null;
    },
    
    /********************************************************************/
    
    /* Splitter event functions */
    
    onMouseDownSplitter: function(event)
    {
        var i,tab,browser;
        
        tileTabs.lastScreenX = event.screenX;
        tileTabs.lastScreenY = event.screenY;
        
        tileTabs.downTile = tileTabs.findTileBySplitter(tileTabs.rootTile,event.currentTarget.id);
        
        for (i = 0; i < tileTabs.downTile.splitterID.length; i++) if (tileTabs.downTile.splitterID[i] == event.currentTarget.id) break;
        tileTabs.downSplitter = i;
        
        if (tileTabs.downTile.type == "hsplit") tileTabs.tabPanels.setAttribute("style","cursor: n-resize;");
        else tileTabs.tabPanels.setAttribute("style","cursor: e-resize;");
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            tab = gBrowser.tabContainer.childNodes[i];
            
            if (tab.hasAttribute("tiletabs-assigned"))
            {
                browser = tab.linkedBrowser;
                browser.style.setProperty("pointer-events","none","");
            }
        }
        
        tileTabs.tabPanels.addEventListener("mousemove",tileTabs.onMouseMoveSplitter,false);
        window.addEventListener("mouseup",tileTabs.onMouseUpSplitter,false);
    },
    
    onMouseMoveSplitter: function(event)
    {
        var c0tile,c1tile,diffX,diffY,height,width;
        
        if (tileTabs.moveTimeout != null) return;  /* allow time for previous resizeTile/drawTile to complete */
        
        tileTabs.moveTimeout = window.setTimeout(function() { tileTabs.moveTimeout = null },10);
        
        c0tile = tileTabs.downTile.childTile[tileTabs.downSplitter];
        c1tile = tileTabs.downTile.childTile[tileTabs.downSplitter+1];
        
        if (tileTabs.downTile.type == "hsplit")  /* horizontal split */
        {
            diffY = tileTabs.lastScreenY-event.screenY;
            height = c0tile.height+c1tile.height;
            
            if (c0tile.height-diffY < tileTabs.splitterSnap && diffY > 0)
            {
                c0tile.ratio = 0;
                c1tile.ratio = height/(tileTabs.downTile.height-tileTabs.downTile.splitterID.length*tileTabs.splitterSize);
                tileTabs.resizeTile(c0tile,c0tile.width,0,c0tile.left,c0tile.top);
                tileTabs.resizeTile(c1tile,c1tile.width,height,c1tile.left,c0tile.top+tileTabs.splitterSize);
            }
            else if (c1tile.height+diffY < tileTabs.splitterSnap && diffY < 0)
            {
                c0tile.ratio = height/(tileTabs.downTile.height-tileTabs.downTile.splitterID.length*tileTabs.splitterSize);
                c1tile.ratio = 0;
                tileTabs.resizeTile(c1tile,c1tile.width,0,c1tile.left,c0tile.top+height+tileTabs.splitterSize);
                tileTabs.resizeTile(c0tile,c0tile.width,height,c0tile.left,c0tile.top);
            }
            else
            {
                c0tile.ratio = (c0tile.height-diffY)/(tileTabs.downTile.height-tileTabs.downTile.splitterID.length*tileTabs.splitterSize);
                c1tile.ratio = (c1tile.height+diffY)/(tileTabs.downTile.height-tileTabs.downTile.splitterID.length*tileTabs.splitterSize);
                tileTabs.resizeTile(c0tile,c0tile.width,c0tile.height-diffY,c0tile.left,c0tile.top);
                tileTabs.resizeTile(c1tile,c1tile.width,c1tile.height+diffY,c1tile.left,c1tile.top-diffY);
            }
        }
        else  /* vertical split */
        {
            diffX = tileTabs.lastScreenX-event.screenX;
            width = c0tile.width+c1tile.width; 
            
            if (c0tile.width-diffX < tileTabs.splitterSnap && diffX > 0)
            {
                c0tile.ratio = 0;
                c1tile.ratio = width/(tileTabs.downTile.width-tileTabs.downTile.splitterID.length*tileTabs.splitterSize);
                tileTabs.resizeTile(c0tile,0,c0tile.height,c0tile.left,c0tile.top);
                tileTabs.resizeTile(c1tile,width,c1tile.height,c0tile.left+tileTabs.splitterSize,c1tile.top);
            }
            else if (c1tile.width+diffX < tileTabs.splitterSnap && diffX < 0)
            {
                c0tile.ratio = width/(tileTabs.downTile.width-tileTabs.downTile.splitterID.length*tileTabs.splitterSize);
                c1tile.ratio = 0;
                tileTabs.resizeTile(c1tile,0,c1tile.height,c0tile.left+width+tileTabs.splitterSize,c1tile.top);
                tileTabs.resizeTile(c0tile,width,c0tile.height,c0tile.left,c0tile.top);
            }
            else
            {
                c0tile.ratio = (c0tile.width-diffX)/(tileTabs.downTile.width-tileTabs.downTile.splitterID.length*tileTabs.splitterSize);
                c1tile.ratio = (c1tile.width+diffX)/(tileTabs.downTile.width-tileTabs.downTile.splitterID.length*tileTabs.splitterSize);
                tileTabs.resizeTile(c0tile,c0tile.width-diffX,c0tile.height,c0tile.left,c0tile.top);
                tileTabs.resizeTile(c1tile,c1tile.width+diffX,c1tile.height,c1tile.left-diffX,c1tile.top);
            }
        }
        
        tileTabs.drawTile(tileTabs.downTile);
        tileTabs.highlightPanel();
        
        tileTabs.lastScreenX = event.screenX;
        tileTabs.lastScreenY = event.screenY;
    },
    
    onMouseUpSplitter: function(event)
    {
        var i,tab,browser;
        
        tileTabs.tabPanels.removeAttribute("style");
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            tab = gBrowser.tabContainer.childNodes[i];
            
            if (tab.hasAttribute("tiletabs-assigned"))
            {
                browser = tab.linkedBrowser;
                browser.style.removeProperty("pointer-events");
            }
        }
        
        tileTabs.tabPanels.removeEventListener("mousemove",tileTabs.onMouseMoveSplitter,false);
        window.removeEventListener("mouseup",tileTabs.onMouseUpSplitter,false);
        
        tileTabs.saveRestoreLayout();
    },
    
    onDblClickSplitter: function(event)
    {
        var tile;
    
        tile = tileTabs.findTileBySplitter(tileTabs.rootTile,event.currentTarget.id);
        
        tileTabs.equalizeTile(tile);
        
        tileTabs.saveRestoreLayout();
        
        tileTabs.drawTile(tile);
        tileTabs.highlightPanel();
    },
    
    /********************************************************************/
    
    /* Menu event functions */
    
    onPopupShowingMainMenu: function(event)
    {
        var i,recentFlag,savedFlag,child,tabNodes,subMenuPopup,subMenuItem;
        var recentNames = new Array();
        
        tileTabs.verifyRecentNames();
        
        recentFlag = (tileTabs.prefs.getCharPref("layout-recentnames") != "");
        savedFlag = (tileTabs.prefs.getChildList("layout-string-",{}).length > 0);
        
        if (event.originalTarget.id == "tiletabs-mainmenu-popup")  /* first level menu popup */
        {
            document.getElementById("tiletabs-mainmenu-new").setAttribute("disabled","false");
            document.getElementById("tiletabs-mainmenu-recent").setAttribute("disabled",!recentFlag);
            document.getElementById("tiletabs-mainmenu-open").setAttribute("disabled",!savedFlag);
            document.getElementById("tiletabs-mainmenu-close").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-save").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-delete").setAttribute("disabled",!savedFlag);
            document.getElementById("tiletabs-mainmenu-setdefault").setAttribute("disabled","false");
            document.getElementById("tiletabs-mainmenu-tile").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-tilenew").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-tiledup").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-assign").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-untile").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-expand").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-sync").setAttribute("disabled","true");
            document.getElementById("tiletabs-mainmenu-view").setAttribute("disabled","true");
            
            if (tileTabs.panelCount > 0)  /* layout */
            {
                document.getElementById("tiletabs-mainmenu-close").setAttribute("disabled","false");
                document.getElementById("tiletabs-mainmenu-view").setAttribute("disabled","false");
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)  /* layout visible */
                {
                    document.getElementById("tiletabs-mainmenu-save").setAttribute("disabled","false");
                    document.getElementById("tiletabs-mainmenu-tile").setAttribute("disabled","false");
                    document.getElementById("tiletabs-mainmenu-tilenew").setAttribute("disabled","false");
                    document.getElementById("tiletabs-mainmenu-tiledup").setAttribute("disabled","false");
                    document.getElementById("tiletabs-mainmenu-assign").setAttribute("disabled","false");
                    document.getElementById("tiletabs-mainmenu-untile").setAttribute("disabled","false");
                    document.getElementById("tiletabs-mainmenu-expand").setAttribute("disabled","false");
                    document.getElementById("tiletabs-mainmenu-sync").setAttribute("disabled","false");
                }
            }
            else  /* no layout */
            {
                if (tileTabs.ffVersion >= "4.0" && tileTabs.prefs.getBoolPref("menu-tabgroup")) tabNodes = gBrowser.visibleTabs;
                else tabNodes = gBrowser.tabContainer.childNodes;
                if (tabNodes.length > 1)  /* several tabs */
                {
                    document.getElementById("tiletabs-mainmenu-tile").setAttribute("disabled","false");
                }
                document.getElementById("tiletabs-mainmenu-tilenew").setAttribute("disabled","false");
                document.getElementById("tiletabs-mainmenu-tiledup").setAttribute("disabled","false");
            }
            
            document.getElementById("tiletabs-mainmenu-sync").setAttribute("checked",tileTabs.syncScroll);
            document.getElementById("tiletabs-mainmenu-view").setAttribute("checked",tileTabs.viewTiled);
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level2")  /* second level menu popup */
        {
            if (event.originalTarget.parentNode.id.substr(event.originalTarget.parentNode.id.length-6) == "recent")
            {
                subMenuPopup = event.originalTarget;
                
                for (i = subMenuPopup.childNodes.length-1; i >= 0; i--)
                {
                    child = subMenuPopup.childNodes[i];
                    subMenuPopup.removeChild(child);
                }
                
                recentNames = tileTabs.prefs.getCharPref("layout-recentnames").split(",");
                for (i = 0; i < recentNames.length; i++)
                {
                    subMenuItem = document.createElement("menuitem");
                    subMenuPopup.appendChild(subMenuItem);
                    subMenuItem.setAttribute("label",recentNames[i]);
                    subMenuItem.setAttribute("oncommand","tileTabs.menuActions('recent','" + recentNames[i] + "')");
                }
            }
            else if (event.originalTarget.parentNode.id.substr(event.originalTarget.parentNode.id.length-10) == "setdefault")
            {
                document.getElementById("tiletabs-mainmenu-setdefault-saved").setAttribute("disabled",!savedFlag);
                
                document.getElementById("tiletabs-mainmenu-setdefault-" + tileTabs.prefs.getCharPref("layout-defaulttype")).setAttribute("checked","true");
            }
            else if (event.originalTarget.parentNode.id.substr(event.originalTarget.parentNode.id.length-6) == "assign")
            {
                tileTabs.showTabList(event.originalTarget,"assign",false);
            }
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level3")  /* third level menu popup */
        {
            tileTabs.showTabList(event.originalTarget,event.originalTarget.parentNode.id.substr(18),true);
        }
    },
    
    onPopupShowingAppMenu: function(event)
    {
        var i,recentFlag,savedFlag,child,tabNodes,subMenuPopup,subMenuItem;
        var recentNames = new Array();
        
        tileTabs.verifyRecentNames();
        
        recentFlag = (tileTabs.prefs.getCharPref("layout-recentnames") != "");
        savedFlag = (tileTabs.prefs.getChildList("layout-string-",{}).length > 0);
        
        if (event.originalTarget.id == "tiletabs-appmenu-popup")  /* first level menu popup */
        {
            document.getElementById("tiletabs-appmenu-new").setAttribute("disabled","false");
            document.getElementById("tiletabs-appmenu-recent").setAttribute("disabled",!recentFlag);
            document.getElementById("tiletabs-appmenu-open").setAttribute("disabled",!savedFlag);
            document.getElementById("tiletabs-appmenu-close").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-save").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-delete").setAttribute("disabled",!savedFlag);
            document.getElementById("tiletabs-appmenu-setdefault").setAttribute("disabled","false");
            document.getElementById("tiletabs-appmenu-tile").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-tilenew").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-tiledup").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-assign").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-untile").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-expand").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-sync").setAttribute("disabled","true");
            document.getElementById("tiletabs-appmenu-view").setAttribute("disabled","true");
            
            if (tileTabs.panelCount > 0)  /* layout */
            {
                document.getElementById("tiletabs-appmenu-close").setAttribute("disabled","false");
                document.getElementById("tiletabs-appmenu-view").setAttribute("disabled","false");
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)  /* layout visible */
                {
                    document.getElementById("tiletabs-appmenu-save").setAttribute("disabled","false");
                    document.getElementById("tiletabs-appmenu-tile").setAttribute("disabled","false");
                    document.getElementById("tiletabs-appmenu-tilenew").setAttribute("disabled","false");
                    document.getElementById("tiletabs-appmenu-tiledup").setAttribute("disabled","false");
                    document.getElementById("tiletabs-appmenu-assign").setAttribute("disabled","false");
                    document.getElementById("tiletabs-appmenu-untile").setAttribute("disabled","false");
                    document.getElementById("tiletabs-appmenu-expand").setAttribute("disabled","false");
                    document.getElementById("tiletabs-appmenu-sync").setAttribute("disabled","false");
                }
            }
            else  /* no layout */
            {
                if (tileTabs.ffVersion >= "4.0" && tileTabs.prefs.getBoolPref("menu-tabgroup")) tabNodes = gBrowser.visibleTabs;
                else tabNodes = gBrowser.tabContainer.childNodes;
                if (tabNodes.length > 1)  /* several tabs */
                {
                    document.getElementById("tiletabs-appmenu-tile").setAttribute("disabled","false");
                }
                document.getElementById("tiletabs-appmenu-tilenew").setAttribute("disabled","false");
                document.getElementById("tiletabs-appmenu-tiledup").setAttribute("disabled","false");
            }
            
            document.getElementById("tiletabs-appmenu-sync").setAttribute("checked",tileTabs.syncScroll);
            document.getElementById("tiletabs-appmenu-view").setAttribute("checked",tileTabs.viewTiled);
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level2")  /* second level menu popup */
        {
            if (event.originalTarget.parentNode.id.substr(event.originalTarget.parentNode.id.length-6) == "recent")
            {
                subMenuPopup = event.originalTarget;
                
                for (i = subMenuPopup.childNodes.length-1; i >= 0; i--)
                {
                    child = subMenuPopup.childNodes[i];
                    subMenuPopup.removeChild(child);
                }
                
                recentNames = tileTabs.prefs.getCharPref("layout-recentnames").split(",");
                for (i = 0; i < recentNames.length; i++)
                {
                    subMenuItem = document.createElement("menuitem");
                    subMenuPopup.appendChild(subMenuItem);
                    subMenuItem.setAttribute("label",recentNames[i]);
                    subMenuItem.setAttribute("oncommand","tileTabs.menuActions('recent','" + recentNames[i] + "')");
                }
            }
            else if (event.originalTarget.parentNode.id.substr(event.originalTarget.parentNode.id.length-10) == "setdefault")
            {
                document.getElementById("tiletabs-appmenu-setdefault-saved").setAttribute("disabled",!savedFlag);
                
                document.getElementById("tiletabs-appmenu-setdefault-" + tileTabs.prefs.getCharPref("layout-defaulttype")).setAttribute("checked","true");
            }
            else if (event.originalTarget.parentNode.id.substr(event.originalTarget.parentNode.id.length-6) == "assign")
            {
                tileTabs.showTabList(event.originalTarget,"assign",false);
            }
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level3")  /* third level menu popup */
        {
            tileTabs.showTabList(event.originalTarget,event.originalTarget.parentNode.id.substr(17));
        }
    },
    
    onPopupShowingTabMenu: function(event)
    {
        var hide,menu,tabNodes;
        
        if ((tileTabs.ffVersion == "3.6" && event.originalTarget.getAttribute("anonid") == "tabContextMenu") ||
            (tileTabs.ffVersion >= "4.0" && event.originalTarget.id == "tabContextMenu"))  /* first level menu popup */
        {
            if (document.popupNode.localName == "tab")  /* right-click on tab */
            {
                hide = !tileTabs.prefs.getBoolPref("menu-tilesubmenu");
                
                document.getElementById("tiletabs-tabmenu-tile").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-tile"));
                document.getElementById("tiletabs-tabmenu-tilenew").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-tilenew"));
                document.getElementById("tiletabs-tabmenu-tiledup").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-tiledup"));
                document.getElementById("tiletabs-tabmenu-assign").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-assign"));
                document.getElementById("tiletabs-tabmenu-untile").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-untile"));
                document.getElementById("tiletabs-tabmenu-expand").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-expand"));
                
                document.getElementById("tiletabs-tabsubmenu").setAttribute("hidden",hide);
                document.getElementById("tiletabs-tabsubmenu-tile").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-tile"));
                document.getElementById("tiletabs-tabsubmenu-tilenew").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-tilenew"));
                document.getElementById("tiletabs-tabsubmenu-tiledup").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-tiledup"));
                document.getElementById("tiletabs-tabsubmenu-assign").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-assign"));
                document.getElementById("tiletabs-tabsubmenu-untile").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-untile"));
                document.getElementById("tiletabs-tabsubmenu-expand").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-expand"));
                
                if (tileTabs.prefs.getBoolPref("menu-tilesubmenu")) menu = "tabsubmenu"; else menu = "tabmenu";
                
                document.getElementById("tiletabs-" + menu + "-tile").setAttribute("disabled","true");
                document.getElementById("tiletabs-" + menu + "-tilenew").setAttribute("disabled","true");
                document.getElementById("tiletabs-" + menu + "-tiledup").setAttribute("disabled","true");
                document.getElementById("tiletabs-" + menu + "-assign").setAttribute("disabled","true");
                document.getElementById("tiletabs-" + menu + "-untile").setAttribute("disabled","true");
                document.getElementById("tiletabs-" + menu + "-expand").setAttribute("disabled","true");
                
                if (tileTabs.panelCount > 0)  /* layout */
                {
                    if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)  /* layout visible */
                    {
                        if (document.popupNode.getAttribute("tiletabs-assigned") != "true") document.getElementById("tiletabs-" + menu + "-tile").setAttribute("disabled","false");
                        document.getElementById("tiletabs-" + menu + "-tilenew").setAttribute("disabled","false");
                        document.getElementById("tiletabs-" + menu + "-tiledup").setAttribute("disabled","false");
                        if (document.popupNode != gBrowser.selectedTab) document.getElementById("tiletabs-" + menu + "-assign").setAttribute("disabled","false");
                        if (document.popupNode.getAttribute("tiletabs-assigned") == "true") document.getElementById("tiletabs-" + menu + "-untile").setAttribute("disabled","false");
                        if (document.popupNode.getAttribute("tiletabs-assigned") == "true") document.getElementById("tiletabs-" + menu + "-expand").setAttribute("disabled","false");
                    }
                }
                else  /* no layout */
                {
                    if (tileTabs.ffVersion >= "4.0" && tileTabs.prefs.getBoolPref("menu-tabgroup")) tabNodes = gBrowser.visibleTabs;
                    else tabNodes = gBrowser.tabContainer.childNodes;
                    if (tabNodes.length > 1)  /* several tabs */
                    {
                        if (document.popupNode != gBrowser.selectedTab) document.getElementById("tiletabs-" + menu + "-tile").setAttribute("disabled","false");
                    }
                    document.getElementById("tiletabs-" + menu + "-tilenew").setAttribute("disabled","false");
                    document.getElementById("tiletabs-" + menu + "-tiledup").setAttribute("disabled","false");
                }
            }
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level2")  /* second level menu popup */
        {
        }
    },
    
    onPopupShowingContentMenu: function(event)
    {
        var i,hide,menu,tabNodes;
        
        if (event.originalTarget.id == "contentAreaContextMenu")  /* first level menu popup */
        {
            hide = !tileTabs.prefs.getBoolPref("menu-tilesubmenu");
            
            document.getElementById("tiletabs-contentmenu-tile").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-tile"));
            document.getElementById("tiletabs-contentmenu-tilenew").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-tilenew"));
            document.getElementById("tiletabs-contentmenu-tiledup").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-tiledup"));
            document.getElementById("tiletabs-contentmenu-tilelink").setAttribute("hidden","true");
            document.getElementById("tiletabs-contentmenu-assign").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-assign"));
            document.getElementById("tiletabs-contentmenu-untile").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-untile"));
            document.getElementById("tiletabs-contentmenu-expand").setAttribute("hidden",!hide || !tileTabs.prefs.getBoolPref("menu-expand"));

            document.getElementById("tiletabs-contentsubmenu").setAttribute("hidden",hide);
            document.getElementById("tiletabs-contentsubmenu-tile").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-tile"));
            document.getElementById("tiletabs-contentsubmenu-tilenew").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-tilenew"));
            document.getElementById("tiletabs-contentsubmenu-tiledup").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-tiledup"));
            document.getElementById("tiletabs-contentsubmenu-tilelink").setAttribute("hidden","true");
            document.getElementById("tiletabs-contentsubmenu-assign").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-assign"));
            document.getElementById("tiletabs-contentsubmenu-untile").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-untile"));
            document.getElementById("tiletabs-contentsubmenu-expand").setAttribute("hidden",hide || !tileTabs.prefs.getBoolPref("menu-expand"));
            
            if (tileTabs.prefs.getBoolPref("menu-tilesubmenu")) menu = "contentsubmenu"; else menu = "contentmenu";
            
            document.getElementById("tiletabs-" + menu + "-tile").setAttribute("disabled","true");
            document.getElementById("tiletabs-" + menu + "-tilenew").setAttribute("disabled","true");
            document.getElementById("tiletabs-" + menu + "-tiledup").setAttribute("disabled","true");
            document.getElementById("tiletabs-" + menu + "-assign").setAttribute("disabled","true");
            document.getElementById("tiletabs-" + menu + "-untile").setAttribute("disabled","true");
            document.getElementById("tiletabs-" + menu + "-expand").setAttribute("disabled","true");
            
            if (tileTabs.panelCount > 0)  /* layout */
            {
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)  /* layout visible */
                {
                    document.getElementById("tiletabs-" + menu + "-tile").setAttribute("disabled","false");
                    document.getElementById("tiletabs-" + menu + "-tilenew").setAttribute("disabled","false");
                    document.getElementById("tiletabs-" + menu + "-tiledup").setAttribute("disabled","false");
                    if (gContextMenu.onLink) document.getElementById("tiletabs-" + menu + "-tilelink").setAttribute("hidden",!tileTabs.prefs.getBoolPref("menu-tilelink"));
                    document.getElementById("tiletabs-" + menu + "-assign").setAttribute("disabled","false");
                    document.getElementById("tiletabs-" + menu + "-untile").setAttribute("disabled","false");
                    document.getElementById("tiletabs-" + menu + "-expand").setAttribute("disabled","false");
                }
            }
            else  /* no layout */
            {
                if (tileTabs.ffVersion >= "4.0" && tileTabs.prefs.getBoolPref("menu-tabgroup")) tabNodes = gBrowser.visibleTabs;
                else tabNodes = gBrowser.tabContainer.childNodes;
                if (tabNodes.length > 1)  /* several tabs */
                {
                    document.getElementById("tiletabs-" + menu + "-tile").setAttribute("disabled","false");
                }
                document.getElementById("tiletabs-" + menu + "-tilenew").setAttribute("disabled","false");
                document.getElementById("tiletabs-" + menu + "-tiledup").setAttribute("disabled","false");
                if (gContextMenu.onLink) document.getElementById("tiletabs-" + menu + "-tilelink").setAttribute("hidden",!tileTabs.prefs.getBoolPref("menu-tilelink"));
            }
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level2")  /* second level menu popup */
        {
            if (event.originalTarget.parentNode.id.substr(event.originalTarget.parentNode.id.length-6) == "assign")
            {
                tileTabs.showTabList(event.originalTarget,"assign",false);
            }
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level3")  /* third level menu popup */
        {
            if (tileTabs.prefs.getBoolPref("menu-tilesubmenu")) i = 24; else i = 21;
            tileTabs.showTabList(event.originalTarget,event.originalTarget.parentNode.id.substr(i),true);
        }
    },
    
    onPopupShowingSplitterMenu: function(event)
    {
        var tile,action;
        
        if (event.originalTarget.id == "tiletabs-splittermenu-popup")  /* first level menu popup */
        {
            tile = tileTabs.findTileBySplitter(tileTabs.rootTile,document.popupNode.id);
            tileTabs.highlightGroup(tile,true);
            
            if (document.popupNode.localName == "vbox")  /* right-click on vertical splitter */
            {
                document.getElementById("tiletabs-splittermenu-equalizegroup-horiz").setAttribute("hidden",false);
                document.getElementById("tiletabs-splittermenu-equalizegroup-vert").setAttribute("hidden",true);
            }
            else  /* right-click on horizontal splitter */
            {
                document.getElementById("tiletabs-splittermenu-equalizegroup-horiz").setAttribute("hidden",true);
                document.getElementById("tiletabs-splittermenu-equalizegroup-vert").setAttribute("hidden",false);
            }
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level2")  /* second level menu popup */
        {
            action = event.originalTarget.parentNode.id.substr(event.originalTarget.parentNode.id.lastIndexOf("-")+1);
            if (document.popupNode.localName == "vbox")  /* right-click on vertical splitter */
            {
                document.getElementById("tiletabs-splittermenu-" + action + "-abovegroup").setAttribute("hidden",false);
                document.getElementById("tiletabs-splittermenu-" + action + "-leftgroup").setAttribute("hidden",true);
                document.getElementById("tiletabs-splittermenu-" + action + "-rightgroup").setAttribute("hidden",true);
                document.getElementById("tiletabs-splittermenu-" + action + "-belowgroup").setAttribute("hidden",false);
            }
            else  /* right-click on horizontal splitter */
            {
                document.getElementById("tiletabs-splittermenu-" + action + "-abovegroup").setAttribute("hidden",true);
                document.getElementById("tiletabs-splittermenu-" + action + "-leftgroup").setAttribute("hidden",false);
                document.getElementById("tiletabs-splittermenu-" + action + "-rightgroup").setAttribute("hidden",false);
                document.getElementById("tiletabs-splittermenu-" + action + "-belowgroup").setAttribute("hidden",true);
            }
        }
        else if (event.originalTarget.getAttribute("class") == "tiletabs-level3")  /* third level menu popup */
        {
            tileTabs.showTabList(event.originalTarget,event.originalTarget.parentNode.id.substr(22),true);
        }
    },
    
    onPopupHidingSplitterMenu: function(event)
    {
        var tile;
        
        if (event.originalTarget.id == "tiletabs-splittermenu-popup")  /* first level menu popup */
        {
            tile = tileTabs.findTileBySplitter(tileTabs.rootTile,document.popupNode.id);
            tileTabs.highlightGroup(tile,false);
        }
    },
    
    onPopupShowingButtonMenu: function(event)
    {
        var i,recentFlag,savedFlag,child,tabNodes,subMenuPopup,subMenuItem;
        var recentNames = new Array();
        
        tileTabs.verifyRecentNames();
        
        recentFlag = (tileTabs.prefs.getCharPref("layout-recentnames") != "");
        savedFlag = (tileTabs.prefs.getChildList("layout-string-",{}).length > 0);
        
        if (event.id == "tiletabs-buttonmenu-popup")  /* first level menu popup */
        {
            document.getElementById("tiletabs-buttonmenu-new").setAttribute("disabled","false");
            document.getElementById("tiletabs-buttonmenu-recent").setAttribute("disabled",!recentFlag);
            document.getElementById("tiletabs-buttonmenu-open").setAttribute("disabled",!savedFlag);
            document.getElementById("tiletabs-buttonmenu-close").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-save").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-delete").setAttribute("disabled",!savedFlag);
            document.getElementById("tiletabs-buttonmenu-setdefault").setAttribute("disabled","false");
            document.getElementById("tiletabs-buttonmenu-tile").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-tilenew").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-tiledup").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-assign").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-untile").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-expand").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-sync").setAttribute("disabled","true");
            document.getElementById("tiletabs-buttonmenu-view").setAttribute("disabled","true");
            
            if (tileTabs.panelCount > 0)  /* layout */
            {
                document.getElementById("tiletabs-buttonmenu-close").setAttribute("disabled","false");
                document.getElementById("tiletabs-buttonmenu-view").setAttribute("disabled","false");
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)  /* layout visible */
                {
                    document.getElementById("tiletabs-buttonmenu-save").setAttribute("disabled","false");
                    document.getElementById("tiletabs-buttonmenu-tile").setAttribute("disabled","false");
                    document.getElementById("tiletabs-buttonmenu-tilenew").setAttribute("disabled","false");
                    document.getElementById("tiletabs-buttonmenu-tiledup").setAttribute("disabled","false");
                    document.getElementById("tiletabs-buttonmenu-assign").setAttribute("disabled","false");
                    document.getElementById("tiletabs-buttonmenu-untile").setAttribute("disabled","false");
                    document.getElementById("tiletabs-buttonmenu-expand").setAttribute("disabled","false");
                    document.getElementById("tiletabs-buttonmenu-sync").setAttribute("disabled","false");
                }
            }
            else  /* no layout */
            {
                if (tileTabs.ffVersion >= "4.0" && tileTabs.prefs.getBoolPref("menu-tabgroup")) tabNodes = gBrowser.visibleTabs;
                else tabNodes = gBrowser.tabContainer.childNodes;
                if (tabNodes.length > 1)  /* several tabs */
                {
                    document.getElementById("tiletabs-buttonmenu-tile").setAttribute("disabled","false");
                }
                document.getElementById("tiletabs-buttonmenu-tilenew").setAttribute("disabled","false");
                document.getElementById("tiletabs-buttonmenu-tiledup").setAttribute("disabled","false");
            }
            
            document.getElementById("tiletabs-buttonmenu-sync").setAttribute("checked",tileTabs.syncScroll);
            document.getElementById("tiletabs-buttonmenu-view").setAttribute("checked",tileTabs.viewTiled);
        }
        else if (event.getAttribute("class") == "tiletabs-level2")  /* second level menu popup */
        {
            if (event.parentNode.id.substr(event.parentNode.id.length-6) == "recent")
            {
                subMenuPopup = event;
                
                for (i = subMenuPopup.childNodes.length-1; i >= 0; i--)
                {
                    child = subMenuPopup.childNodes[i];
                    subMenuPopup.removeChild(child);
                }
                
                recentNames = tileTabs.prefs.getCharPref("layout-recentnames").split(",");
                for (i = 0; i < recentNames.length; i++)
                {
                    subMenuItem = document.createElement("menuitem");
                    subMenuPopup.appendChild(subMenuItem);
                    subMenuItem.setAttribute("label",recentNames[i]);
                    subMenuItem.setAttribute("oncommand","tileTabs.menuActions('recent','" + recentNames[i] + "')");
                }
            }
            else if (event.parentNode.id.substr(event.parentNode.id.length-10) == "setdefault")
            {
                document.getElementById("tiletabs-buttonmenu-setdefault-saved").setAttribute("disabled",!savedFlag);
                
                document.getElementById("tiletabs-buttonmenu-setdefault-" + tileTabs.prefs.getCharPref("layout-defaulttype")).setAttribute("checked","true");
            }
            else if (event.parentNode.id.substr(event.parentNode.id.length-6) == "assign")
            {
                tileTabs.showTabList(event,"assign",false);
            }
        }
        else if (event.getAttribute("class") == "tiletabs-level3")  /* third level menu popup */
        {
            tileTabs.showTabList(event,event.parentNode.id.substr(20),true);
        }
    },
    
    verifyRecentNames: function()
    {
        var i;
        var recentNames = new Array();
        
        recentNames = tileTabs.prefs.getCharPref("layout-recentnames").split(",");
        if (recentNames[0] == "") recentNames.length = 0;
        for (i = 0; i < recentNames.length; i++)
        {
            if (!tileTabs.prefs.prefHasUserValue("layout-string-" + recentNames[i]))
            {
                recentNames.splice(i,1);
                i--;
            }
        }
        if (recentNames.length > 10) recentNames.length = 10;                
        tileTabs.prefs.setCharPref("layout-recentnames",recentNames.join(","));
    },
    
    showTabList: function(menuPopup,action,tileCommand)
    {
        var i,j,child,tab,menuItem;
        
        for (i = menuPopup.childNodes.length-1; i >= 0; i--)
        {
            child = menuPopup.childNodes[i];
            menuPopup.removeChild(child);
        }
        
        if (tileTabs.ffVersion >= "4.0")
        {
            for (j = 0; j < gBrowser.visibleTabs.length; j++)
            {
                tab = gBrowser.visibleTabs[j];
                
                menuItem = document.createElement("menuitem");
                menuPopup.appendChild(menuItem);
                menuItem.setAttribute("class","menuitem-iconic");
                menuItem.setAttribute("image",tab.getAttribute("image"));
                menuItem.setAttribute("label",tab.label);
                menuItem.setAttribute("oncommand","tileTabs.menuActions('" + action + "'," + tab._tPos + ")");
                if (tab == gBrowser.selectedTab)
                {
                    menuItem.setAttribute("disabled","true");
                    menuItem.setAttribute("label","[\u2009" + tab.label + "\u2009]");
                }
                if (tab.getAttribute("tiletabs-assigned") == "true")
                {
                    if (tileCommand) menuItem.setAttribute("disabled","true");
                    if (tileCommand) menuItem.setAttribute("label","[\u2009" + tab.label + "\u2009]");
                    menuItem.style.setProperty("color",tileTabs.skinColor,"");
                }
            }
            if (!tileTabs.prefs.getBoolPref("menu-tabgroup") && gBrowser.visibleTabs.length < gBrowser.tabContainer.childNodes.length)
            {
                menuItem = document.createElement("menuseparator");
                menuPopup.appendChild(menuItem);
            }
        }
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            tab = gBrowser.tabContainer.childNodes[i];
            
            if (tileTabs.ffVersion >= "4.0")
            {
                if (tileTabs.prefs.getBoolPref("menu-tabgroup")) continue;
                for (j = 0; j < gBrowser.visibleTabs.length; j++) if (tab == gBrowser.visibleTabs[j]) break;
                if ( j < gBrowser.visibleTabs.length) continue;
            }
            
            menuItem = document.createElement("menuitem");
            menuPopup.appendChild(menuItem);
            menuItem.setAttribute("class","menuitem-iconic");
            menuItem.setAttribute("image",tab.getAttribute("image"));
            menuItem.setAttribute("label",tab.label);
            menuItem.setAttribute("oncommand","tileTabs.menuActions('" + action + "'," + i + ")");
            if (tab == gBrowser.selectedTab)
            {
                menuItem.setAttribute("disabled","true");
                menuItem.setAttribute("label","[\u2009" + tab.label + "\u2009]");
            }
            if (tab.getAttribute("tiletabs-assigned") == "true")
            {
                if (tileCommand) menuItem.setAttribute("disabled","true");
                if (tileCommand) menuItem.setAttribute("label","[\u2009" + tab.label + "\u2009]");
                menuItem.style.setProperty("color",tileTabs.skinColor,"");
            }
        }
    },
    
    /********************************************************************/
    
    /* Toolbar button click event function */
    
    onClickButton: function(event)
    {
        if (event.button == 0 && event.detail == 1)  /* single left-click */
        {
            if (tileTabs.panelCount == 0)  /* no tile layout */
            {
                tileTabs.menuActions('new-default',null);
            }
            else  /* tile layout */
            {
                tileTabs.menuActions('view',null);
            }
        }
        else if (event.button == 0 && event.detail > 1)  /* double left-click */
        {
            tileTabs.menuActions('close',null);
        }
        else if (event.button == 1 && event.detail == 1)  /* single middle-click */
        {
            if (tileTabs.panelCount == 0)  /* no tile layout */
            {
                tileTabs.menuActions('new-default',null);
            }
            else  /* tile layout */
            {
                tileTabs.menuActions('close',null);
            }
        }
    },
    
    /********************************************************************/
    
    /* Menu action function */
    
    menuActions: function(action,param)
    {
        var i,j,tab,tile,buttonmenu,tabNodes,cols,rows,uri,element,srctile,notificationBox,browser;
        var arguments = new Object;
        var recentNames = new Array();
        var postData = { };
        
        switch (action)
        {
            case "open":
            
                arguments.layoutname = "";
                window.openDialog("chrome://tiletabs/content/tiletabs-openlayout.xul","","chrome,dialog,modal,titlebar,centerscreen",arguments);
                if (arguments.layoutname == "") break;
                param = arguments.layoutname;
            
            case "new-default":
            
                if (action == "new-default" && tileTabs.prefs.getCharPref("layout-defaulttype") == "saved") param = tileTabs.prefs.getCharPref("layout-defaultname");
            
            case "recent":
            
                recentNames = tileTabs.prefs.getCharPref("layout-recentnames").split(",");
                if (recentNames[0] == "") recentNames.length = 0;
                for (i = 0; i < recentNames.length; i++)
                    if (recentNames[i] == param)
                    {
                        recentNames.splice(i,1);
                        i--;
                    }
                recentNames.unshift(param);
                if (recentNames.length > 10) recentNames.length = 10;                
                tileTabs.prefs.setCharPref("layout-recentnames",recentNames.join(","));
            
            case "new-2vert":
            case "new-2horiz":
            case "new-3vert":
            case "new-3horiz":
            case "new-4vertgrid":
            case "new-4horizgrid":
            case "new-allvert":
            case "new-allhoriz":
            case "new-allvertgrid":
            case "new-allhorizgrid":
            
            case "close":
            
                for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
                {
                    tab = gBrowser.tabContainer.childNodes[i];
                    if (tab.getAttribute("tiletabs-assigned") == "true")
                    {
                        if (tileTabs.panelCount > 1)
                        {
                            tile = tileTabs.findTileByPanel(tileTabs.rootTile,tab.linkedPanel);
                            tileTabs.removeTile(tile,false);
                            if (tileTabs.unassignTab(tab)) i--;
                        }
                        else  /* exit tile mode */
                        {
                            tileTabs.removeTileLast();
                            if (tileTabs.unassignTab(tab)) i--;
                            
                            tileTabs.deleteRestoreLayout();
                            
                            tileTabs.syncScroll = false;
                            tileTabs.viewTiled = false;
                            
                            if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
                            {
                                buttonmenu.removeAttribute("layout");
                                document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").removeAttribute("checked");
                            }
                            
                            tileTabs.hideNonSelectedPanels();
                            tileTabs.drawSelectedPanel();
                        }
                    }
                }
                tileTabs.styleTiledTabs();
                
                if (action == "close") break;
                
                if (action == "open" || action == "recent") tileTabs.openNamedLayout(param);
                else if (action == "new-default" && tileTabs.prefs.getCharPref("layout-defaulttype") == "saved")
                {
                    if (!tileTabs.openNamedLayout(param))
                    {
                        /* saved name does not exist - revert to 2vert layout */
                        
                        tileTabs.addTileFirst(gBrowser.selectedTab);
                        tileTabs.addTile(tileTabs.rootTile,tileTabs.assignTab(),"right");
                            
                        tileTabs.prefs.setCharPref("layout-defaulttype","2vert")
                        tileTabs.prefs.setCharPref("layout-defaultname","")
                    }
                }
                else
                {
                    if (action == "new-default") action = "new-" + tileTabs.prefs.getCharPref("layout-defaulttype");
                    
                    switch (action.substr(action.indexOf("-")+1))  /* new layout */
                    {
                        case "2vert":
                            tileTabs.addTileFirst(gBrowser.selectedTab);
                            tileTabs.addTile(tileTabs.rootTile,tileTabs.assignTab(),"right");
                            break;
                        
                        case "2horiz":
                            tileTabs.addTileFirst(gBrowser.selectedTab);
                            tileTabs.addTile(tileTabs.rootTile,tileTabs.assignTab(),"below");
                            break;
                        
                        case "3vert":
                            tileTabs.addTileFirst(gBrowser.selectedTab);
                            tileTabs.addTile(tileTabs.rootTile,tileTabs.assignTab(),"right");
                            tileTabs.addTile(tileTabs.rootTile.childTile[0],tileTabs.assignTab(),"right");
                            tileTabs.equalizeTile(tileTabs.rootTile);
                            break;
                        
                        case "3horiz":
                            tileTabs.addTileFirst(gBrowser.selectedTab);
                            tileTabs.addTile(tileTabs.rootTile,tileTabs.assignTab(),"below");
                            tileTabs.addTile(tileTabs.rootTile.childTile[0],tileTabs.assignTab(),"below");
                            tileTabs.equalizeTile(tileTabs.rootTile);
                            break;
                        
                        case "4vertgrid":
                            tileTabs.addTileFirst(gBrowser.selectedTab);
                            tileTabs.addTile(tileTabs.rootTile,tileTabs.assignTab(),"right");
                            tileTabs.addTile(tileTabs.rootTile.childTile[0],tileTabs.assignTab(),"below");
                            tileTabs.addTile(tileTabs.rootTile.childTile[1],tileTabs.assignTab(),"below");
                            break;
                        
                        case "4horizgrid":
                            tileTabs.addTileFirst(gBrowser.selectedTab);
                            tileTabs.addTile(tileTabs.rootTile,tileTabs.assignTab(),"below");
                            tileTabs.addTile(tileTabs.rootTile.childTile[0],tileTabs.assignTab(),"right");
                            tileTabs.addTile(tileTabs.rootTile.childTile[1],tileTabs.assignTab(),"right");
                            break;
                        
                        case "allvert":
                            if (tileTabs.ffVersion >= "4.0") tabNodes = gBrowser.visibleTabs;
                            else tabNodes = gBrowser.tabContainer.childNodes;
                            if (tabNodes.length <= 1) return;
                            tileTabs.addTileFirst(tabNodes[0]);
                            for (i = 1; i < tabNodes.length; i++)
                            {
                                tile = tileTabs.findTileByPanel(tileTabs.rootTile,tabNodes[i-1].linkedPanel);
                                tileTabs.addTile(tile,tabNodes[i],"right");
                            }
                            tileTabs.equalizeTile(tileTabs.rootTile);
                            break;
                        
                        case "allhoriz":
                            if (tileTabs.ffVersion >= "4.0") tabNodes = gBrowser.visibleTabs;
                            else tabNodes = gBrowser.tabContainer.childNodes;
                            if (tabNodes.length <= 1) return;
                            tileTabs.addTileFirst(tabNodes[0]);
                            for (i = 1; i < tabNodes.length; i++)
                            {
                                tile = tileTabs.findTileByPanel(tileTabs.rootTile,tabNodes[i-1].linkedPanel);
                                tileTabs.addTile(tile,tabNodes[i],"below");
                            }
                            tileTabs.equalizeTile(tileTabs.rootTile);
                            break;
                        
                        case "allvertgrid":
                            if (tileTabs.ffVersion >= "4.0") tabNodes = gBrowser.visibleTabs;
                            else tabNodes = gBrowser.tabContainer.childNodes;
                            if (tabNodes.length <= 1) return;
                            tileTabs.addTileFirst(tabNodes[0]);
                            cols = Math.ceil(Math.sqrt(tabNodes.length));
                            rows = Math.ceil(tabNodes.length/cols);
                            for (i = 1; i < cols; i++)
                            {
                                tile = tileTabs.findTileByPanel(tileTabs.rootTile,tabNodes[(i-1)*rows].linkedPanel);
                                tileTabs.addTile(tile,tabNodes[i*rows],"right");
                            }
                            for (i = 0; i < cols; i++)
                            {
                                for (j = 1; j < rows && i*rows+j < tabNodes.length; j++)
                                {
                                    tile = tileTabs.findTileByPanel(tileTabs.rootTile,tabNodes[i*rows+(j-1)].linkedPanel);
                                    tileTabs.addTile(tile,tabNodes[i*rows+j],"below");
                                }
                            }
                            tileTabs.equalizeTile(tileTabs.rootTile);
                            for (i = 0; i < cols; i++)
                            {
                                tile = tileTabs.findTileByPanel(tileTabs.rootTile,tabNodes[i*rows].linkedPanel);
                                tileTabs.equalizeTile(tile.parentTile);
                            }
                            break;
                        
                        case "allhorizgrid":
                            if (tileTabs.ffVersion >= "4.0") tabNodes = gBrowser.visibleTabs;
                            else tabNodes = gBrowser.tabContainer.childNodes;
                            if (tabNodes.length <= 1) return;
                            tileTabs.addTileFirst(tabNodes[0]);
                            rows = Math.ceil(Math.sqrt(tabNodes.length));
                            cols = Math.ceil(tabNodes.length/rows);
                            for (i = 1; i < rows; i++)
                            {
                                tile = tileTabs.findTileByPanel(tileTabs.rootTile,tabNodes[(i-1)*cols].linkedPanel);
                                tileTabs.addTile(tile,tabNodes[i*cols],"below");
                            }
                            for (i = 0; i < rows; i++)
                            {
                                for (j = 1; j < cols && i*cols+j < tabNodes.length; j++)
                                {
                                    tile = tileTabs.findTileByPanel(tileTabs.rootTile,tabNodes[i*cols+(j-1)].linkedPanel);
                                    tileTabs.addTile(tile,tabNodes[i*cols+j],"right");
                                }
                            }
                            tileTabs.equalizeTile(tileTabs.rootTile);
                            for (i = 0; i < rows; i++)
                            {
                                tile = tileTabs.findTileByPanel(tileTabs.rootTile,tabNodes[i*cols].linkedPanel);
                                tileTabs.equalizeTile(tile.parentTile);
                            }
                            break;
                    }
                }
                
                tileTabs.prevTiledTab = gBrowser.selectedTab;
                
                tileTabs.saveRestoreLayout();
                
                tileTabs.viewTiled = true;
                
                if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
                {
                    buttonmenu.setAttribute("layout","true");
                    document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").setAttribute("checked","true");
                }
                
                tileTabs.hideNonTiledPanels();
                tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                tileTabs.highlightPanel();
                
                tileTabs.styleTiledTabs();
                tileTabs.moveTiledTabs();
                break;
            
            case "save":
            
                arguments.layoutname = "";
                arguments.layouttabs = tileTabs.prefs.getBoolPref("layout-savetabs"); 
                window.openDialog("chrome://tiletabs/content/tiletabs-savelayout.xul","","chrome,dialog,modal,titlebar,centerscreen",arguments);
                if (arguments.layoutname == "") break;
                
                tileTabs.saveNamedLayout(arguments.layoutname,arguments.layouttabs);
                tileTabs.prefs.setBoolPref("layout-savetabs",arguments.layouttabs); 
                
                recentNames = tileTabs.prefs.getCharPref("layout-recentnames").split(",");
                if (recentNames[0] == "") recentNames.length = 0;
                for (i = 0; i < recentNames.length; i++)
                    if (recentNames[i] == arguments.layoutname)
                    {
                        recentNames.splice(i,1);
                        i--;
                    }
                recentNames.unshift(arguments.layoutname);
                if (recentNames.length > 10) recentNames.length = 10;                
                tileTabs.prefs.setCharPref("layout-recentnames",recentNames.join(","));
                break;
            
            case "delete":
            
                arguments.layoutname = "";
                window.openDialog("chrome://tiletabs/content/tiletabs-deletelayout.xul","","chrome,dialog,modal,titlebar,centerscreen",arguments);
                if (arguments.layoutname == "") break;
                
                tileTabs.deleteNamedLayout(arguments.layoutname);
                
                recentNames = tileTabs.prefs.getCharPref("layout-recentnames").split(",");
                if (recentNames[0] == "") recentNames.length = 0;
                for (i = 0; i < recentNames.length; i++)
                    if (recentNames[i] == arguments.layoutname)
                    {
                        recentNames.splice(i,1);
                        i--;
                    }
                tileTabs.prefs.setCharPref("layout-recentnames",recentNames.join(","));
                
                if (arguments.layoutname == tileTabs.prefs.getCharPref("layout-defaultname"))
                {
                    tileTabs.prefs.setCharPref("layout-defaulttype","2vert")
                    tileTabs.prefs.setCharPref("layout-defaultname","")
                }
                break;
            
            case "setdefault":
            
                if (param == "saved")
                {
                    arguments.layoutname = "";
                    if (tileTabs.prefs.getCharPref("layout-defaulttype") == "saved") arguments.layoutname = tileTabs.prefs.getCharPref("layout-defaultname");
                    window.openDialog("chrome://tiletabs/content/tiletabs-defaultlayout.xul","","chrome,dialog,modal,titlebar,centerscreen",arguments);
                    if (arguments.layoutname == "") break;
                    
                    tileTabs.prefs.setCharPref("layout-defaultname",arguments.layoutname);
                }
                tileTabs.prefs.setCharPref("layout-defaulttype",param);
                break;
            
            case "tile-above":
            case "tile-left":
            case "tile-right":
            case "tile-below":
            case "tilenew-above":
            case "tilenew-left":
            case "tilenew-right":
            case "tilenew-below":
            case "tiledup-above":
            case "tiledup-left":
            case "tiledup-right":
            case "tiledup-below":
            case "tilelink-above":
            case "tilelink-left":
            case "tilelink-right":
            case "tilelink-below":
            
            case "tile-abovegroup":
            case "tile-leftgroup":
            case "tile-rightgroup":
            case "tile-belowgroup":
            case "tilenew-abovegroup":
            case "tilenew-leftgroup":
            case "tilenew-rightgroup":
            case "tilenew-belowgroup":
            case "tiledup-abovegroup":
            case "tiledup-leftgroup":
            case "tiledup-rightgroup":
            case "tiledup-belowgroup":
            
                switch (action.substr(0,action.indexOf("-")))
                {
                    case "tile":
                        if (param == null) tab = document.popupNode;  /* right-click on tab */
                        else tab = gBrowser.tabContainer.childNodes[param];  /* tab number from menu list */
                        break;
                    
                    case "tilenew":
                        if (tileTabs.prefs.getBoolPref("tab-loadhome")) uri = gHomeButton.getHomePage().split("|",1);
                        else uri = "about:blank";
                        tab = gBrowser.loadOneTab(uri,null,null,null,true,false);
                        break;
                    
                    case "tiledup":
                        if (param == null) tab = document.popupNode;  /* right-click on tab */
                        else /* param == -1 */ tab = gBrowser.selectedTab;  /* currently selected tab */
                        uri = gBrowser.getBrowserForTab(tab).currentURI.spec;
                        tab = gBrowser.loadOneTab(uri,null,null,null,true,false);
                        break;
                    
                    case "tilelink":
                        element = document.popupNode;
                        while (element.href == null && element.parentNode != null) element = element.parentNode;
                        if (element.href != null)
                        {
                            try { urlSecurityCheck(element.href,element.ownerDocument.nodePrincipal); }
                            catch (e) { return; }
                            
                            uri = getShortcutOrURI(element.href,postData);
                            if (!uri) return;
                            
                            tab = gBrowser.loadOneTab(uri,null,null,postData.value,true,false);
                        }
                        else
                        {
                            uri = gBrowser.getBrowserForTab(gBrowser.selectedTab).currentURI.spec;
                            tab = gBrowser.loadOneTab(uri,null,null,null,true,false);
                        }
                        break;
                }
                
                if (action.indexOf("group") >= 0)
                {
                    tile = tileTabs.findTileBySplitter(tileTabs.rootTile,document.popupNode.id);
                    action = action.substr(0,action.length-5);
                }
                else
                {
                    if (tileTabs.panelCount > 0)
                    {
                        tile = tileTabs.findTileByPanel(tileTabs.rootTile,gBrowser.selectedTab.linkedPanel);
                    }
                    else
                    {
                        tileTabs.addTileFirst(gBrowser.selectedTab);
                        tileTabs.prevTiledTab = gBrowser.selectedTab;
                        tile = tileTabs.rootTile;
                        
                        if (tileTabs.tabMove) gBrowser.moveTabTo(gBrowser.selectedTab,tileTabs.panelCount-1);
                        
                        tileTabs.viewTiled = true;
                        
                        if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
                        {
                            buttonmenu.setAttribute("layout","true");
                            document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").setAttribute("checked","true");
                        }
                    }
                }
                
                tileTabs.addTile(tile,tab,action.substr(action.indexOf("-")+1));
                
                if (tileTabs.tabMove) gBrowser.moveTabTo(tab,tileTabs.panelCount-1);
                
                tileTabs.saveRestoreLayout();
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                {
                    tileTabs.hideNonTiledPanels();
                    tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                    tileTabs.highlightPanel();
                }
                else
                {
                    tileTabs.hideNonSelectedPanels();
                    tileTabs.drawSelectedPanel();
                }
                tileTabs.styleTiledTabs();
                break;
            
            case "assign":
            
                if (param == null) tab = document.popupNode;  /* right-click on tab */
                else tab = gBrowser.tabContainer.childNodes[param];  /* tab number from menu list */
                
                tile = tileTabs.findTileByPanel(tileTabs.rootTile,gBrowser.selectedTab.linkedPanel);
                srctile = tileTabs.findTileByPanel(tileTabs.rootTile,tab.linkedPanel);
                
                if (tab.getAttribute("tiletabs-assigned") == "true")
                {
                    /* Attach tab to active tile */
                    
                    tile.panelID = tab.linkedPanel;
                    
                    /* Attach selected tab to source tile */
                    
                    srctile.panelID = gBrowser.selectedTab.linkedPanel;
                }
                else
                {
                    /* Detach selected tab from active tile */
                    
                    notificationBox = document.getElementById(gBrowser.selectedTab.linkedPanel);
                    notificationBox.removeEventListener("click",tileTabs.onClickPanel,false);
                    notificationBox.removeEventListener("mousedown",tileTabs.onMouseDownPanel,false);
                    
                    browser = gBrowser.selectedTab.linkedBrowser;
                    browser.removeEventListener("scroll",tileTabs.onScrollBrowser,false);
                    
                    gBrowser.selectedTab.removeAttribute("tiletabs-assigned");
                    
                    tileTabs.unassignTab(gBrowser.selectedTab);
                    
                    /* Attach tab to active tile */
                    
                    tile.panelID = tab.linkedPanel;
                    
                    notificationBox = document.getElementById(tab.linkedPanel);
                    notificationBox.addEventListener("click",tileTabs.onClickPanel,false);
                    notificationBox.addEventListener("mousedown",tileTabs.onMouseDownPanel,false);
                    
                    browser = tab.linkedBrowser;
                    browser.addEventListener("scroll",tileTabs.onScrollBrowser,false);
                    
                    tab.setAttribute("tiletabs-assigned","true");
                }
                
                tileTabs.prevTiledTab = tab;
                gBrowser.selectedTab = tileTabs.prevTiledTab;
                
                tileTabs.saveRestoreLayout();
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                {
                    tileTabs.hideNonTiledPanels();
                    tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                    tileTabs.highlightPanel();
                }
                else
                {
                    tileTabs.hideNonSelectedPanels();
                    tileTabs.drawSelectedPanel();
                }
                tileTabs.styleTiledTabs();
                tileTabs.moveTiledTabs();
                break;
            
            case "untile":
            
                if (param == null) tab = document.popupNode;  /* right-click on tab */
                else /* param == -1 */ tab = gBrowser.selectedTab;  /* currently selected tab */
                
                tile = tileTabs.findTileByPanel(tileTabs.rootTile,tab.linkedPanel);
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                {
                    if (tab == gBrowser.selectedTab) tileTabs.removeTile(tile,true);
                    else tileTabs.removeTile(tile,false);
                }
                else tileTabs.removeTile(tile,false);
                tileTabs.unassignTab(tab);
                
                if (tileTabs.tabMove) gBrowser.moveTabTo(tab,tileTabs.panelCount);
                
                if (tileTabs.panelCount > 1)  /* if right-click on tab, selected tab may not have changed */
                {
                    tileTabs.saveRestoreLayout();
                    
                    if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                    {
                        tileTabs.hideNonTiledPanels();
                        tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                        tileTabs.highlightPanel();
                    }
                }
                else  /* exit tile mode */
                {
                    tab = tileTabs.findTabByPanel(tileTabs.rootTile.panelID);
                    tileTabs.removeTileLast();
                    tileTabs.unassignTab(tab);
                    
                    tileTabs.deleteRestoreLayout();
                    
                    tileTabs.syncScroll = false;
                    tileTabs.viewTiled = false;
                    
                    if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
                    {
                        buttonmenu.removeAttribute("layout");
                        document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").removeAttribute("checked");
                    }
                    
                    tileTabs.hideNonSelectedPanels();
                    tileTabs.drawSelectedPanel();
                }
                tileTabs.styleTiledTabs();
                break;
            
            case "expand":
            
                if (param == null) tab = document.popupNode;  /* right-click on tab */
                else /* param == -1 */ tab = gBrowser.selectedTab;  /* currently selected tab */
                
                tile = tileTabs.findTileByPanel(tileTabs.rootTile,tab.linkedPanel);
                
                while (tile.parentTile != null && tile.ratio == 1)
                {
                    tile = tile.parentTile;
                }
                
                if (tile.parentTile != null)
                {
                    tile.ratio = 1;
                    for (i = 0; i < tile.parentTile.childTile.length; i++)
                    {
                        if (tile.parentTile.childTile[i] != tile)
                        {
                            tile.parentTile.childTile[i].ratio = 0;
                        }
                    }
                    tileTabs.resizeTile(tile.parentTile,tile.parentTile.width,tile.parentTile.height,tile.parentTile.left,tile.parentTile.top);
                }
                else
                {
                    tile = tileTabs.findTileByPanel(tileTabs.rootTile,tab.linkedPanel);
                    
                    while (tile.parentTile != null)
                    {
                        tileTabs.equalizeTile(tile.parentTile);
                        tile = tile.parentTile;
                    }
                }
                
                tileTabs.prevTiledTab = tab;
                gBrowser.selectedTab = tileTabs.prevTiledTab;
                
                tileTabs.saveRestoreLayout();
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                {    
                    tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                    tileTabs.highlightPanel();
                }
                else
                {
                    tileTabs.drawSelectedPanel();
                }
                break;
            
            case "equalizegroup":
            
                tile = tileTabs.findTileBySplitter(tileTabs.rootTile,document.popupNode.id);
                
                tileTabs.equalizeTile(tile);
                
                tileTabs.saveRestoreLayout();
                
                tileTabs.drawTile(tile);
                tileTabs.highlightPanel();
                break;
            
            case "sync":
            
                tileTabs.syncScroll = !tileTabs.syncScroll;
                break;
            
            case "view":
            
                tileTabs.viewTiled = !tileTabs.viewTiled;
                
                if (tileTabs.viewTiled)
                {
                    if (tileTabs.prefs.getBoolPref("tab-keepselect"))
                    {
                        if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") != "true")
                        {
                            tile = tileTabs.findTileByPanel(tileTabs.rootTile,tileTabs.prevTiledTab.linkedPanel);
                            
                            /* Detach previous tab from active tile */
                            
                            notificationBox = document.getElementById(tileTabs.prevTiledTab.linkedPanel);
                            notificationBox.removeEventListener("click",tileTabs.onClickPanel,false);
                            notificationBox.removeEventListener("mousedown",tileTabs.onMouseDownPanel,false);
                            
                            browser = tileTabs.prevTiledTab.linkedBrowser;
                            browser.removeEventListener("scroll",tileTabs.onScrollBrowser,false);
                            
                            tileTabs.prevTiledTab.removeAttribute("tiletabs-assigned");
                            
                            tileTabs.unassignTab(tileTabs.prevTiledTab);
                            
                            /* Attach selected tab to active tile */
                            
                            tile.panelID = gBrowser.selectedTab.linkedPanel;
                            
                            notificationBox = document.getElementById(gBrowser.selectedTab.linkedPanel);
                            notificationBox.addEventListener("click",tileTabs.onClickPanel,false);
                            notificationBox.addEventListener("mousedown",tileTabs.onMouseDownPanel,false);
                            
                            browser = gBrowser.selectedTab.linkedBrowser;
                            browser.addEventListener("scroll",tileTabs.onScrollBrowser,false);
                            
                            gBrowser.selectedTab.setAttribute("tiletabs-assigned","true");
                        }
                        
                        tileTabs.prevTiledTab = gBrowser.selectedTab;
                        
                        tileTabs.saveRestoreLayout();
                    }
                    else gBrowser.selectedTab = tileTabs.prevTiledTab;
                    
                    
                    if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
                        document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").setAttribute("checked","true");
                }
                else
                {
                    if ((buttonmenu = document.getElementById("tiletabs-buttonmenu")) != null)  /* button on toolbar */
                        document.getAnonymousElementByAttribute(buttonmenu,"anonid","button").removeAttribute("checked");
                }
                
                if (gBrowser.selectedTab.getAttribute("tiletabs-assigned") == "true" && tileTabs.viewTiled)
                {
                    tileTabs.hideNonTiledPanels();
                    tileTabs.drawTile(tileTabs.rootTile);  /* removes existing highlight */
                    tileTabs.highlightPanel();
                }
                else
                {
                    tileTabs.hideNonSelectedPanels();
                    tileTabs.drawSelectedPanel();
                }
                tileTabs.styleTiledTabs();
                break;
        }    
    },
    
    /********************************************************************/
    
    /* Save, Open and Delete named layout functions */
    
    saveNamedLayout: function(layoutName,layoutTabs)
    {
        if (layoutTabs) tileTabs.layoutType = 1;  /* save layout and assigned tab urls */
        else tileTabs.layoutType = 0;  /* save layout */
        
        tileTabs.layoutString = tileTabs.layoutType + ";";
        tileTabs.layoutString += tileTabs.panelCount + ";";
        
        tileTabs.createLayoutString(tileTabs.rootTile);
        
        tileTabs.prefs.setCharPref("layout-string-" + layoutName,tileTabs.layoutString);
    },
    
    openNamedLayout: function(layoutName)
    {
        var j;
        
        if (!tileTabs.prefs.prefHasUserValue("layout-string-" + layoutName)) return false;
        
        tileTabs.layoutString = tileTabs.prefs.getCharPref("layout-string-" + layoutName);
        
        j = tileTabs.layoutString.indexOf(";");
        tileTabs.layoutType = +tileTabs.layoutString.substr(0,j);
        tileTabs.layoutString = tileTabs.layoutString.substr(j+1);
        
        j = tileTabs.layoutString.indexOf(";");
        tileTabs.layoutString = tileTabs.layoutString.substr(j+1);
        
        tileTabs.applyLayoutString(null,false);
        
        tileTabs.resizeTile(tileTabs.rootTile,tileTabs.tabPanels.boxObject.width,tileTabs.tabPanels.boxObject.height,0,0);
        
        return true;
    },
    
    deleteNamedLayout: function(layoutName)
    {
        tileTabs.prefs.clearUserPref("layout-string-" + layoutName);
    },
    
    /********************************************************************/
    
    /* Save, Open and Delete restore layout functions */
    
    saveRestoreLayout: function()
    {
        tileTabs.layoutType = 2;  /* save layout and assigned tab positions */
        
        tileTabs.layoutString = tileTabs.layoutType + ";";
        tileTabs.layoutString += tileTabs.panelCount + ";";
        
        tileTabs.createLayoutString(tileTabs.rootTile);
        
        tileTabs.sstore.setWindowValue(window,"tiletabs-restore",tileTabs.layoutString);
    },
    
    openRestoreLayout: function()
    {
        var j;
        
        tileTabs.layoutString = tileTabs.sstore.getWindowValue(window,"tiletabs-restore");
        
        if (tileTabs.layoutString == "") return false;
        
        j = tileTabs.layoutString.indexOf(";");
        tileTabs.layoutType = +tileTabs.layoutString.substr(0,j);
        tileTabs.layoutString = tileTabs.layoutString.substr(j+1);
        
        j = tileTabs.layoutString.indexOf(";");
        if (+tileTabs.layoutString.substr(0,j) > gBrowser.tabContainer.childNodes.length)
        {
            tileTabs.deleteRestoreLayout();
            alert("Tile Tabs:\n - fewer tabs than tiles\n - cannot restore tile layout");
            return false;
        }
        tileTabs.layoutString = tileTabs.layoutString.substr(j+1);
        
        tileTabs.applyLayoutString(null,false);
        
        tileTabs.resizeTile(tileTabs.rootTile,tileTabs.tabPanels.boxObject.width,tileTabs.tabPanels.boxObject.height,0,0);
        
        return true;
    },
    
    deleteRestoreLayout: function()
    {
        tileTabs.sstore.deleteWindowValue(window,"tiletabs-restore");
    },
    
    /********************************************************************/
    
    /* Create and Apply layout string functions */
    
    createLayoutString: function(tile)
    {
        var i;
        
        if (tile.type == "panel")  /* panel tile */
        {
            if (tileTabs.layoutType == 0) tileTabs.layoutString += "panel" + "," + tile.ratio.toFixed(4) + "," + "-1" + ";";
            else if (tileTabs.layoutType == 1) tileTabs.layoutString += "panel" + "," + tile.ratio.toFixed(4) + "," + tileTabs.findTabByPanel(tile.panelID).linkedBrowser.currentURI.spec + ";";
            else if (tileTabs.layoutType == 2) tileTabs.layoutString += "panel" + "," + tile.ratio.toFixed(4) + "," + tileTabs.findTabByPanel(tile.panelID)._tPos + ";";
        }
        else  /* split tile */
        {
            tileTabs.layoutString += tile.type + "," + tile.ratio.toFixed(4) + "," + tile.childTile.length + ";";
            
            for (i = 0; i < tile.childTile.length; i++)
                tileTabs.createLayoutString(tile.childTile[i]);
        }
    },
    
    applyLayoutString: function(parentTile,firstChild)
    {
        var i,j,tile,splitterName,splitterBox,menuPopup,tab,notificationBox,browser,childCount;
        
        tile = new tileTabs.Tile;
        tile.parentTile = parentTile;
        if (parentTile == null) tileTabs.rootTile = tile;
        else 
        {
            tile.parentTile.childTile[tile.parentTile.childTile.length] = tile;
            
            if (!firstChild)
            {
                tile.parentTile.splitterID[tile.parentTile.splitterID.length] = splitterName = "tiletabs-splitter" + tileTabs.splitterCount++;
                
                /* Create splitter */
                
                splitterBox = (tile.parentTile.type == "vsplit") ? document.createElement("vbox") : document.createElement("hbox");
                splitterBox.setAttribute("id",splitterName);
                splitterBox.setAttribute("context","tiletabs-splittermenu-popup");
                tileTabs.extraBox.appendChild(splitterBox);
                
                /* Add listeners for splitter */
                
                splitterBox.addEventListener("dblclick",tileTabs.onDblClickSplitter,false);
                splitterBox.addEventListener("mousedown",tileTabs.onMouseDownSplitter,false);
                
                /* Add listeners for popup menu */
                
                menuPopup = document.getElementById("tiletabs-splittermenu-popup");
                menuPopup.addEventListener("popupshowing",tileTabs.onPopupShowingSplitterMenu,false);
                menuPopup.addEventListener("popuphiding",tileTabs.onPopupHidingSplitterMenu,false);
            }
        }
        
        if (tileTabs.layoutString.substr(0,5) == "panel")  /* panel tile */
        {
            tile.type = "panel";
            tileTabs.layoutString = tileTabs.layoutString.substr(6);
            
            j = tileTabs.layoutString.indexOf(",");
            tile.ratio = +tileTabs.layoutString.substr(0,j);
            tileTabs.layoutString = tileTabs.layoutString.substr(j+1);
            
            j = tileTabs.layoutString.indexOf(";");
            if (tileTabs.layoutType == 0)
            {
                if (tileTabs.panelCount == 0) tab = gBrowser.selectedTab;
                else tab = tileTabs.assignTab();
            }
            else if (tileTabs.layoutType == 1)
            {
                for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
                {
                    tab = gBrowser.tabContainer.childNodes[i];
                    if (tab.getAttribute("tiletabs-assigned") != "true" && tab.linkedBrowser.currentURI.spec == tileTabs.layoutString.substr(0,j)) break;
                }
                if (i == gBrowser.tabContainer.childNodes.length) tab = gBrowser.loadOneTab(tileTabs.layoutString.substr(0,j),null,null,null,true,false);
                if (tileTabs.panelCount == 0) gBrowser.selectedTab = tab;
            }
            else if (tileTabs.layoutType == 2)
            {
                i = +tileTabs.layoutString.substr(0,j);
                if (i < gBrowser.tabContainer.childNodes.length && gBrowser.tabContainer.childNodes[i].getAttribute("tiletabs-assigned") != "true")
                {
                    tab = gBrowser.tabContainer.childNodes[i];
                    tab.linkedBrowser.reload();  /* in case tab is in hidden tabgroup and would not normally be loaded */
                }
                else tab = gBrowser.loadOneTab("about:blank",null,null,null,true,false);
            }
            tile.panelID = tab.linkedPanel;
            tileTabs.layoutString = tileTabs.layoutString.substr(j+1);
            
            /* Add listeners for tab.linkedPanel and associated browser */
            
            notificationBox = document.getElementById(tab.linkedPanel);
            notificationBox.addEventListener("click",tileTabs.onClickPanel,false);
            notificationBox.addEventListener("mousedown",tileTabs.onMouseDownPanel,false);
            
            browser = tab.linkedBrowser;
            browser.addEventListener("scroll",tileTabs.onScrollBrowser,false);
            
            /* Set tile attribute for tab */
            
            tab.setAttribute("tiletabs-assigned","true");
            
            tileTabs.panelCount++;
        }
        else  /* split tile */
        {
            tile.type = tileTabs.layoutString.substr(0,6);
            tileTabs.layoutString = tileTabs.layoutString.substr(7);
            
            j = tileTabs.layoutString.indexOf(",");
            tile.ratio = +tileTabs.layoutString.substr(0,j);
            tileTabs.layoutString = tileTabs.layoutString.substr(j+1);
            
            j = tileTabs.layoutString.indexOf(";");
            childCount = +tileTabs.layoutString.substr(0,j);
            tileTabs.layoutString = tileTabs.layoutString.substr(j+1);
            
            /* Recursively create tile tree */
            
            for (i = 0; i < childCount; i++)
                tileTabs.applyLayoutString(tile,(i == 0));
        }
    },
    
    /********************************************************************/
    
    /* Debug functions */
    
    debugMessage: function(module,information)
    {
        var info;
        
        tileTabs.debugCount++;
        info = document.getElementById("tiletabs-debug-info");
        info.label = tileTabs.debugCount + " - " + module + " - " + information;
    },
    
    debugInspector: function(button)
    {
        var i,info,tile;
        
        tileTabs.debugCount++;
        tile = tileTabs.debugTile;
        info = document.getElementById("tiletabs-debug-info");
        
        if (tileTabs.rootTile == null) { info.label = "No root tile"; return; }
        
        if (tile != null) tileTabs.highlightGroup(tile,false);
        
        switch (button)
        {
            case "root":
                tile = tileTabs.rootTile;
                break;
            case "up":
                if (tile.parentTile != null) tile = tile.parentTile;
                break;
            case "down":
                if (tile.childTile.length > 0) tile = tile.childTile[0];
                break;
            case "prev":
                if (tile.parentTile != null)
                {
                    for (i = 0; i < tile.parentTile.childTile.length; i++) if (tile.parentTile.childTile[i] == tile) break;
                    if (i > 0) tile = tile.parentTile.childTile[i-1];
                }
                break;
            case "next":
                if (tile.parentTile != null)
                {
                    for (i = 0; i < tile.parentTile.childTile.length; i++) if (tile.parentTile.childTile[i] == tile) break;
                    if (i < tile.parentTile.childTile.length-1) tile = tile.parentTile.childTile[i+1];
                }
                break;
        }
        tileTabs.debugTile = tile;
        
        info.label = tileTabs.debugCount + " - ";
        if (tile.parentTile == null) info.label += "root";
        info.label += " T:" + tile.type + " W:" + tile.width + " H:" + tile.height + " L:" + tile.left + " T:" + tile.top + " R:" + tile.ratio.toFixed(3);
        if (tile.panelID == "") info.label += " P:~"; else info.label += " P:" + tile.panelID.substr(14);
        info.label += " X:" + tile.scrollX + " Y:" + tile.scrollY;
        if (tile.splitterID.length == 0) info.label += " S:~";
        else { info.label += " S:" + tile.splitterID[0].substr(17); for (i = 1; i < tile.splitterID.length; i++) info.label += "/" + tile.splitterID[i].substr(17); }
        info.label += " C:" + tile.childTile.length;
        
        tileTabs.highlightGroup(tile,true);
    }
};

/************************************************************************/

/* Start Tile Tabs */

window.addEventListener("load",tileTabs.onLoad,false);

/************************************************************************/
